use [master]
GO

IF OBJECT_ID('RZ_DICOM_HL7') IS NULL
	CREATE DATABASE [RZ_DICOM_HL7]

GO
Use [RZ_DICOM_HL7]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

CREATE TABLE [dbo].[MESSAGES](
	[message_id] [int] IDENTITY(1,1) NOT NULL,
	[message_date_time] [datetime] NULL,
	[message_type] [nvarchar](400) NULL,
	[message_control_id] [nvarchar](400) NULL,
	[sending_application] [nvarchar](400) NULL,
	[sending_facility] [nvarchar](400) NULL,
	[receiving_application] [nvarchar](400) NULL,
	[receiving_facility] [nvarchar](400) NULL,
PRIMARY KEY CLUSTERED 
(
	[message_id] ASC
)WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]


SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE TABLE [dbo].[QUERY_DEFINITION](
    [query_def_id] [int] IDENTITY(1,1) NOT NULL,
    [query_message_id] [int] NULL,
	[query_date_time] [datetime] NULL,
	[query_format_code] [nvarchar](400) NULL,
	[query_priority] [nvarchar](400) NULL,
	[query_id] [nvarchar](400) NULL,
	[quantity_limited_restrict] [nvarchar](400) NULL,
	[who_subject_filter] [nvarchar](400) NULL,
	[what_subject_filter] [nvarchar](400) NULL,
PRIMARY KEY CLUSTERED 
(
	[query_def_id] ASC
)WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]


GO
SET ANSI_PADDING OFF
GO
ALTER TABLE [dbo].[QUERY_DEFINITION]  WITH CHECK ADD  CONSTRAINT [FK_QUERY_DEFINITION_MESSAGES] FOREIGN KEY([query_message_id])
REFERENCES [dbo].[MESSAGES] ([message_id])
ON UPDATE CASCADE
ON DELETE CASCADE
GO
ALTER TABLE [dbo].[QUERY_DEFINITION] CHECK CONSTRAINT [FK_QUERY_DEFINITION_MESSAGES]


SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE TABLE [dbo].[N_PATIENT](
	[id] [int] IDENTITY(1,1) NOT NULL,
	[patient_id] [nvarchar](64) NULL,
	[patient_message_id] [int] NULL,
	[patient_name] [nvarchar](400) NULL,
	[phone_number] [nvarchar](400) NULL,
	[birth_date] [datetime] NULL,
	[sex] [char](1) NULL,
	[address] [nvarchar](400) NULL,
	[ssn] [numeric](18, 0) NULL,
PRIMARY KEY CLUSTERED 
(
	[id] ASC
)WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]

GO
SET ANSI_PADDING OFF
GO
ALTER TABLE [dbo].[N_PATIENT]  WITH CHECK ADD  CONSTRAINT [FK_N_PATIENT_MESSAGES] FOREIGN KEY([patient_message_id])
REFERENCES [dbo].[MESSAGES] ([message_id])
ON UPDATE CASCADE
ON DELETE CASCADE
GO
ALTER TABLE [dbo].[N_PATIENT] CHECK CONSTRAINT [FK_N_PATIENT_MESSAGES]


SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_PADDING ON
GO
CREATE TABLE [dbo].[REPORTS](
	[report_id] [int] IDENTITY(1,1) NOT NULL,
	[report_patient_id] [int] NULL,
	[set_id] [int] NULL,
	[placer_order_no] [nvarchar](400) NULL,
	[fillers_order_no] [nvarchar](400) NULL,
	[universal_service_id] [nvarchar](400) NULL,
	[observation_date_time] [datetime] NULL,
	[ordering_provider] [nvarchar](400) NULL,
	[result_status] [nvarchar](400) NULL,
	[prin_result_interpreter] [nvarchar](400) NULL,
 CONSTRAINT [PK_REPORTS] PRIMARY KEY CLUSTERED 
(
	[report_id] ASC
)WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]

GO
ALTER TABLE [dbo].[REPORTS]  WITH CHECK ADD  CONSTRAINT [FK_REPORTS_N_PATIENT] FOREIGN KEY([report_patient_id])
REFERENCES [dbo].[N_PATIENT] ([id])
ON UPDATE CASCADE
ON DELETE CASCADE
GO
ALTER TABLE [dbo].[REPORTS] CHECK CONSTRAINT [FK_REPORTS_N_PATIENT]


SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE TABLE [dbo].[REPORT_ITEMS](
	[report_item_id] [int] IDENTITY(1,1) NOT NULL,
	[reportitem_report_id] [int] NULL,
	[set_id] [int] NULL,
	[value_type] [nvarchar](400) NULL,
	[observation_identifer] [nvarchar](400) NULL,
	[observation_results] [nvarchar](400) NULL,
	[units] [nvarchar](400) NULL,
	[reference_range] [nvarchar](400) NULL,
	[abnormal_flags] [nvarchar](400) NULL,
	[observ_result_status] [nvarchar](400) NULL,
 CONSTRAINT [PK_REPORT_ITEMS] PRIMARY KEY CLUSTERED 
(
	[report_item_id] ASC
)WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]

GO
ALTER TABLE [dbo].[REPORT_ITEMS]  WITH CHECK ADD  CONSTRAINT [FK_REPORT_ITEMS_REPORTS] FOREIGN KEY([reportitem_report_id])
REFERENCES [dbo].[REPORTS] ([report_id])
ON UPDATE CASCADE
ON DELETE CASCADE
GO
ALTER TABLE [dbo].[REPORT_ITEMS] CHECK CONSTRAINT [FK_REPORT_ITEMS_REPORTS]
GO


SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE TABLE [dbo].[HL7_QUEUE](
	[que_id] [int] IDENTITY(1,1) NOT NULL,
	[que_status] [int] NOT NULL,
	[que_creation_date] [datetime] NULL CONSTRAINT [DF_HL7_QUEUE_que_creation_date]  DEFAULT (getdate()),
	[que_event_type] [nchar](10) NULL,
	[que_start_table] [nvarchar](100) NULL,
	[que_start_field] [nvarchar](100) NULL,
	[que_start_value] [int] NULL,
	[que_update_date] [datetime] NULL,
	[que_data_type] [nchar](10) NULL,
	[que_retry_count] [int] NULL CONSTRAINT [DF_HL7_QUEUE_que_retry_count]  DEFAULT ((0)),
	[que_last_error_code] [int] NULL,
	[que_last_error_message] [nvarchar](400) NULL,
	[que_app_name] [nvarchar](100) NULL,
	[que_app_facility] [nvarchar](100) NULL,
 CONSTRAINT [PK_HL7_QUEUE] PRIMARY KEY CLUSTERED 
(
	[que_id] ASC
)WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]


SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE TRIGGER [dbo].[RZ_MESSAGES_message_id] 
   ON  [dbo].[MESSAGES] 
   AFTER INSERT
AS 
BEGIN
    SET NOCOUNT ON;
    DECLARE @@NewId int;
    DECLARE @@NewType nvarchar(max);
    
    SELECT @@NewId = [message_id] FROM INSERTED
    SET @@NewType = (SELECT [message_type] FROM [dbo].[MESSAGES] WHERE [message_id] = @@NewId)

if (@@NewType = 'QRY^A19') 
    INSERT INTO [dbo].[HL7_QUEUE]
           ([que_status]
           ,[que_creation_date]
           ,[que_event_type]
           ,[que_start_table]
           ,[que_start_field]
           ,[que_start_value]
           ,[que_update_date]
           ,[que_data_type]
           ,[que_retry_count]
           ,[que_last_error_code]
           ,[que_last_error_message])
     VALUES
           (2
           ,GETDATE()
           ,'QRYA19'
           ,'MESSAGES'
           ,'message_id'
           ,@@NewId
           ,GETDATE()
           ,'0'
           ,0
           ,0
           ,'')
else
	if (@@NewType = 'ORU^R01') 
    INSERT INTO [dbo].[HL7_QUEUE]
           ([que_status]
           ,[que_creation_date]
           ,[que_event_type]
           ,[que_start_table]
           ,[que_start_field]
           ,[que_start_value]
           ,[que_update_date]
           ,[que_data_type]
           ,[que_retry_count]
           ,[que_last_error_code]
           ,[que_last_error_message])
     VALUES
           (2
           ,GETDATE()
           ,'ORUR01'
           ,'MESSAGES'
           ,'message_id'
           ,@@NewId
           ,GETDATE()
           ,'0'
           ,0
           ,0
           ,'')

END
GO


CREATE TABLE [dbo].[STORAGE_COMMIT_REQUEST](
	[scm_id] [int] IDENTITY(1,1) NOT NULL,
	[creation_time] [datetime] NOT NULL CONSTRAINT [DF_STORGAE_COMMIT_REQUEST_creation_time]  DEFAULT (getdate()),
	[update_time] [datetime] NULL,
	[scm_source_ae] [nvarchar](64) NOT NULL,
	[scm_transaction_uid] [nvarchar](64) NOT NULL,
	[tasks_queued] [int] NULL,
	[response_retry_count] [int] NOT NULL CONSTRAINT [DF_STORAGE_COMMIT_REQUEST_retry_count]  DEFAULT ((0)),
	[response_status] [int] NULL,
	[scm_kind] [int] NOT NULL CONSTRAINT [DF_STORAGE_COMMIT_REQUEST_scm_kind]  DEFAULT ((0)),
	[task_id] [int] NULL,
 CONSTRAINT [PK_STORGAE_COMMIT_REQUEST] PRIMARY KEY CLUSTERED 
(
	[scm_id] ASC
)WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY],
 CONSTRAINT [IX_STORGAE_COMMIT_REQUEST] UNIQUE NONCLUSTERED 
(
	[scm_transaction_uid] ASC
)WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]

GO
EXEC sys.sp_addextendedproperty @name=N'MS_Description', @value=N'The number of tasks that were created following the incomming commit request' , @level0type=N'SCHEMA',@level0name=N'dbo', @level1type=N'TABLE',@level1name=N'STORAGE_COMMIT_REQUEST', @level2type=N'COLUMN',@level2name=N'tasks_queued'
GO
EXEC sys.sp_addextendedproperty @name=N'MS_Description', @value=N'The task that generated this outgoing commit request' , @level0type=N'SCHEMA',@level0name=N'dbo', @level1type=N'TABLE',@level1name=N'STORAGE_COMMIT_REQUEST', @level2type=N'COLUMN',@level2name=N'task_id'
GO

SET ANSI_NULLS ON

GO

SET QUOTED_IDENTIFIER ON

GO

CREATE TABLE [dbo].[CONFIGURATION](
	[Key] [nvarchar](50) NOT NULL,
	[Value] [nvarchar](50) NOT NULL,
	[Description] [nvarchar](400) NULL,
 CONSTRAINT [PK_CONFIGURATION] PRIMARY KEY CLUSTERED 
(
	[Key] ASC
)WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]

GO
EXEC sys.sp_addextendedproperty @name=N'MS_Description', @value=N'A unique key of the configuration elemet' , @level0type=N'SCHEMA',@level0name=N'dbo', @level1type=N'TABLE',@level1name=N'CONFIGURATION', @level2type=N'COLUMN',@level2name=N'Key'
GO
EXEC sys.sp_addextendedproperty @name=N'MS_Description', @value=N'The value of this entry' , @level0type=N'SCHEMA',@level0name=N'dbo', @level1type=N'TABLE',@level1name=N'CONFIGURATION', @level2type=N'COLUMN',@level2name=N'Value'
GO
EXEC sys.sp_addextendedproperty @name=N'MS_Description', @value=N'A short description of the key and the value and their use' , @level0type=N'SCHEMA',@level0name=N'dbo', @level1type=N'TABLE',@level1name=N'CONFIGURATION', @level2type=N'COLUMN',@level2name=N'Description'
GO

SET ANSI_NULLS ON

GO

SET QUOTED_IDENTIFIER ON

GO

CREATE TABLE [dbo].[DICOM_APPLICATIONS](

	[id] [int] IDENTITY(1,1) NOT NULL,

	[alias] [nvarchar](400) NULL,

	[ae_title] [nvarchar](64) NOT NULL,

	[host] [nvarchar](64) NULL,

	[port] [int] NULL,

 CONSTRAINT [PK_DICOM_APPLICATIONS] PRIMARY KEY CLUSTERED 

(

	[id] ASC

)WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]

) ON [PRIMARY]

GO

CREATE UNIQUE NONCLUSTERED INDEX [IX_DICOM_APPLICATIONS_AE_TITLE] ON [dbo].[DICOM_APPLICATIONS] 

(

	[ae_title] ASC

)WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, SORT_IN_TEMPDB = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]

GO

EXEC sys.sp_addextendedproperty @name=N'MS_Description', @value=N'The AE Title is unique' , @level0type=N'SCHEMA',@level0name=N'dbo', @level1type=N'TABLE',@level1name=N'DICOM_APPLICATIONS', @level2type=N'INDEX',@level2name=N'IX_DICOM_APPLICATIONS_AE_TITLE'

GO

SET ANSI_NULLS ON

GO

SET QUOTED_IDENTIFIER ON

GO

CREATE TABLE [dbo].[STORAGE_COMMIT_REQUEST_INSTANCES](
	[scm_id] [int] NOT NULL,
	[sop_class_uid] [nvarchar](64) NOT NULL,
	[sop_instance_uid] [nvarchar](64) NOT NULL,
	[status] [int] NOT NULL CONSTRAINT [DF_STORAGE_COMMIT_REQUEST_INSTANCES_status]  DEFAULT ((2)),
	[failure_reason] [int] NOT NULL CONSTRAINT [DF_STORAGE_COMMIT_REQUEST_INSTANCES_failure_reason]  DEFAULT ((0)),
 CONSTRAINT [PK_STORAGE_COMMIT_REQUEST_INSTANCES] PRIMARY KEY CLUSTERED 
(
	[scm_id] ASC,
	[sop_class_uid] ASC,
	[sop_instance_uid] ASC
)WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]
GO

CREATE NONCLUSTERED INDEX [IX_STORAGE_COMMIT_REQUEST_INSTANCES_1] ON [dbo].[STORAGE_COMMIT_REQUEST_INSTANCES] 
(
	[sop_class_uid] ASC,
	[sop_instance_uid] ASC
)WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, SORT_IN_TEMPDB = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
GO

SET ANSI_NULLS ON

GO

SET QUOTED_IDENTIFIER ON

GO

CREATE TABLE [dbo].[DICOM_FILES](
	[file_id] [int] IDENTITY(1,1) NOT NULL,
	[creation_time] [datetime] NOT NULL CONSTRAINT [DF_DICOM_FILES_creation_time]  DEFAULT (getdate()),
	[update_time] [datetime] NULL,
	[patient_name] [nvarchar](64) NOT NULL,
	[patient_id] [nvarchar](64) NOT NULL,
	[patient_sex] [nchar](1) NULL,
	[patient_birth_date] [datetime] NULL,
	[study_instance_uid] [nvarchar](64) NOT NULL,
	[study_id] [nvarchar](16) NULL,
	[study_date_time] [datetime] NULL,
	[accession_number] [nvarchar](16) NULL,
	[modality] [nvarchar](16) NULL,
	[series_number] [nvarchar](16) NULL,
	[series_instance_uid] [nvarchar](64) NOT NULL,
	[instance_number] [nvarchar](16) NULL,
	[sop_class_uid] [nvarchar](64) NOT NULL,
	[sop_instance_uid] [nvarchar](64) NOT NULL,
	[source_ae_title] [nvarchar](64) NULL,
	[rel_path] [nvarchar](400) NOT NULL,
	[task_id] [int] NULL,
 CONSTRAINT [PK_DICOM_FILES] PRIMARY KEY CLUSTERED 

(

	[file_id] ASC

)WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]

) ON [PRIMARY]

GO

EXEC sys.sp_addextendedproperty @name=N'MS_Description', @value=N'A unique identifier of the file' , @level0type=N'SCHEMA',@level0name=N'dbo', @level1type=N'TABLE',@level1name=N'DICOM_FILES', @level2type=N'COLUMN',@level2name=N'file_id'

GO

EXEC sys.sp_addextendedproperty @name=N'MS_Description', @value=N'A timestamp when the record was created' , @level0type=N'SCHEMA',@level0name=N'dbo', @level1type=N'TABLE',@level1name=N'DICOM_FILES', @level2type=N'COLUMN',@level2name=N'creation_time'

GO

EXEC sys.sp_addextendedproperty @name=N'MS_Description', @value=N'A timestamp when the record was updated' , @level0type=N'SCHEMA',@level0name=N'dbo', @level1type=N'TABLE',@level1name=N'DICOM_FILES', @level2type=N'COLUMN',@level2name=N'update_time'

GO

EXEC sys.sp_addextendedproperty @name=N'MS_Description', @value=N'Patient Name' , @level0type=N'SCHEMA',@level0name=N'dbo', @level1type=N'TABLE',@level1name=N'DICOM_FILES', @level2type=N'COLUMN',@level2name=N'patient_name'

GO

EXEC sys.sp_addextendedproperty @name=N'MS_Description', @value=N'Patient ID' , @level0type=N'SCHEMA',@level0name=N'dbo', @level1type=N'TABLE',@level1name=N'DICOM_FILES', @level2type=N'COLUMN',@level2name=N'patient_id'

GO

EXEC sys.sp_addextendedproperty @name=N'MS_Description', @value=N'Study Instance UID' , @level0type=N'SCHEMA',@level0name=N'dbo', @level1type=N'TABLE',@level1name=N'DICOM_FILES', @level2type=N'COLUMN',@level2name=N'study_instance_uid'

GO

EXEC sys.sp_addextendedproperty @name=N'MS_Description', @value=N'Series Instance UID' , @level0type=N'SCHEMA',@level0name=N'dbo', @level1type=N'TABLE',@level1name=N'DICOM_FILES', @level2type=N'COLUMN',@level2name=N'series_instance_uid'

GO

EXEC sys.sp_addextendedproperty @name=N'MS_Description', @value=N'SOP Class UID' , @level0type=N'SCHEMA',@level0name=N'dbo', @level1type=N'TABLE',@level1name=N'DICOM_FILES', @level2type=N'COLUMN',@level2name=N'sop_class_uid'

GO

EXEC sys.sp_addextendedproperty @name=N'MS_Description', @value=N'SOP Instance UID' , @level0type=N'SCHEMA',@level0name=N'dbo', @level1type=N'TABLE',@level1name=N'DICOM_FILES', @level2type=N'COLUMN',@level2name=N'sop_instance_uid'

GO

EXEC sys.sp_addextendedproperty @name=N'MS_Description', @value=N'The AE Title that stored the file' , @level0type=N'SCHEMA',@level0name=N'dbo', @level1type=N'TABLE',@level1name=N'DICOM_FILES', @level2type=N'COLUMN',@level2name=N'source_ae_title'

GO

EXEC sys.sp_addextendedproperty @name=N'MS_Description', @value=N'Relative path' , @level0type=N'SCHEMA',@level0name=N'dbo', @level1type=N'TABLE',@level1name=N'DICOM_FILES', @level2type=N'COLUMN',@level2name=N'rel_path'

GO

EXEC sys.sp_addextendedproperty @name=N'MS_Description', @value=N'The task id that process the file' , @level0type=N'SCHEMA',@level0name=N'dbo', @level1type=N'TABLE',@level1name=N'DICOM_FILES', @level2type=N'COLUMN',@level2name=N'task_id'

GO

EXEC sys.sp_addextendedproperty @name=N'MS_Description', @value=N'The DICOM_FILES table holds one record for every DICOM File stored in the system.' , @level0type=N'SCHEMA',@level0name=N'dbo', @level1type=N'TABLE',@level1name=N'DICOM_FILES'

GO


CREATE UNIQUE NONCLUSTERED INDEX [IX_DICOM_FILES_1] ON [dbo].[DICOM_FILES] 

(

	[patient_name] ASC,

	[patient_id] ASC,

	[study_instance_uid] ASC,

	[series_instance_uid] ASC,

	[sop_class_uid] ASC,

	[sop_instance_uid] ASC,

	[task_id] ASC

)WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, SORT_IN_TEMPDB = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]

GO


CREATE NONCLUSTERED INDEX [IX_DICOM_FILES_2] ON [dbo].[DICOM_FILES] 

(

	[sop_class_uid] ASC,

	[sop_instance_uid] ASC

)WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, SORT_IN_TEMPDB = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]

GO



SET ANSI_NULLS ON

GO

SET QUOTED_IDENTIFIER ON

GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE VIEW [dbo].[UNFINISHED_STORAGE_COMMIT_REQUESTS]
AS
SELECT     dbo.STORAGE_COMMIT_REQUEST.scm_id, dbo.STORAGE_COMMIT_REQUEST.scm_source_ae, dbo.DICOM_APPLICATIONS.host, 
                      dbo.DICOM_APPLICATIONS.port, dbo.STORAGE_COMMIT_REQUEST.scm_transaction_uid
FROM         dbo.STORAGE_COMMIT_REQUEST INNER JOIN
                      dbo.DICOM_APPLICATIONS ON dbo.STORAGE_COMMIT_REQUEST.scm_source_ae = dbo.DICOM_APPLICATIONS.ae_title
WHERE     
(dbo.STORAGE_COMMIT_REQUEST.response_status IS NULL OR dbo.STORAGE_COMMIT_REQUEST.response_status <> 0) 
AND (dbo.STORAGE_COMMIT_REQUEST.response_retry_count < (SELECT     CAST(Value AS int) AS Expr1
                            FROM          dbo.CONFIGURATION AS CONFIGURATION_1
                            WHERE      ([Key] = 'scm_max_retry'))) 
AND (dbo.STORAGE_COMMIT_REQUEST.update_time < -
                          ((SELECT     CAST(Value AS float) AS Expr1
                              FROM         dbo.CONFIGURATION
                              WHERE     ([Key] = 'retry_seconds')) / 60 / 60 / 24) + GETDATE()) 
AND (dbo.STORAGE_COMMIT_REQUEST.scm_kind = 1) 
OR (dbo.STORAGE_COMMIT_REQUEST.response_status IS NULL OR dbo.STORAGE_COMMIT_REQUEST.response_status <> 0) 
AND (dbo.STORAGE_COMMIT_REQUEST.response_retry_count = 0) 
AND (dbo.STORAGE_COMMIT_REQUEST.scm_kind = 1)

GO

SET ANSI_NULLS ON

GO

SET QUOTED_IDENTIFIER ON

GO

CREATE VIEW [dbo].[STORAGE_COMMIT_SUCCEEDED_INSTANCES]

AS

SELECT     dbo.STORAGE_COMMIT_REQUEST_INSTANCES.scm_id, dbo.STORAGE_COMMIT_REQUEST_INSTANCES.sop_class_uid, 

                      dbo.STORAGE_COMMIT_REQUEST_INSTANCES.sop_instance_uid

FROM         dbo.STORAGE_COMMIT_REQUEST_INSTANCES INNER JOIN

                      dbo.DICOM_FILES ON dbo.STORAGE_COMMIT_REQUEST_INSTANCES.sop_class_uid = dbo.DICOM_FILES.sop_class_uid AND 

                      dbo.STORAGE_COMMIT_REQUEST_INSTANCES.sop_instance_uid = dbo.DICOM_FILES.sop_instance_uid

GO



SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE VIEW [dbo].[CFIND_IMAGE_VIEW]
AS
SELECT DISTINCT 
                patient_name AS PatientsName, patient_id AS PatientID, 
				study_id AS StudyID, study_date_time AS StudyDate,
				accession_number AS AccessionNumber,
				study_instance_uid AS StudyInstanceUID, modality AS Modality,
				series_number AS SeriesNumber,
				series_instance_uid AS SeriesInstanceUID, 
				instance_number AS InstanceNumber,
                sop_class_uid AS SOPClassUID, sop_instance_uid AS SOPInstanceUID
FROM         dbo.DICOM_FILES

GO


SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE VIEW [dbo].[CFIND_SERIES_VIEW]
AS
SELECT DISTINCT 
                      patient_name AS PatientsName, patient_id AS PatientID, 
				study_id AS StudyID, study_date_time AS StudyDate,
				accession_number AS AccessionNumber,
				study_instance_uid AS StudyInstanceUID, modality AS Modality,
				series_number AS SeriesNumber,
				series_instance_uid AS SeriesInstanceUID
FROM         dbo.DICOM_FILES

GO


SET ANSI_NULLS ON
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE VIEW [dbo].[CFIND_STUDY_VIEW]
AS
SELECT DISTINCT patient_name AS PatientName, patient_id AS PatientID, study_id AS StudyID,
                (SELECT CAST(dbo.DICOM_FILES.study_date_time AS DATE) AS Expr1) AS StudyDate,
                (SELECT CAST(dbo.DICOM_FILES.study_date_time AS TIME) AS Expr1) AS StudyTime, accession_number AS AccessionNumber, study_instance_uid AS StudyInstanceUID, patient_sex AS PatientSex, patient_birth_date AS PatientBirthDate,
                (SELECT CAST(COUNT(SeriesInstanceUID) AS char(12)) AS Expr1
                FROM    dbo.CFIND_SERIES_VIEW
                WHERE (StudyInstanceUID = dbo.DICOM_FILES.study_instance_uid)) AS NumberOfStudyRelatedSeries,
                (SELECT CAST(COUNT(SOPInstanceUID) AS char(12)) AS Expr1
                FROM    dbo.CFIND_IMAGE_VIEW
                WHERE (StudyInstanceUID = dbo.DICOM_FILES.study_instance_uid)) AS NumberOfStudyRelatedInstances,
                (SELECT CAST
                                ((SELECT DISTINCT Modality + '\' AS 'data()'
                                FROM    dbo.CFIND_IMAGE_VIEW AS CFIND_IMAGE_VIEW_1
                                WHERE  (StudyInstanceUID = dbo.DICOM_FILES.study_instance_uid) FOR XML PATH('')) AS nvarchar(16)) AS Expr2) AS ModalitiesInStudy
FROM    dbo.DICOM_FILES

GO



SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE VIEW [dbo].[CFIND_PATIENT_VIEW]
AS
SELECT DISTINCT 
    patient_name AS PatientName, 
	patient_id AS PatientID, 
	patient_sex AS PatientSex, 
	patient_birth_date AS PatientBirthDate
FROM         dbo.DICOM_FILES
GO


SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE VIEW [dbo].[CMOVE_PATIENT_VIEW]
AS
SELECT DISTINCT 
                      dbo.DICOM_FILES.patient_name AS PatientsName, dbo.DICOM_FILES.patient_id AS PatientID, 
                      dbo.CONFIGURATION.Value + '\' + dbo.DICOM_FILES.rel_path AS file_path
FROM         dbo.DICOM_FILES CROSS JOIN
                      dbo.CONFIGURATION
WHERE     (dbo.CONFIGURATION.[Key] = 'root_dir')

GO


SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE VIEW [dbo].[CMOVE_STUDY_VIEW]
AS
SELECT DISTINCT 
                patient_name AS PatientsName, patient_id AS PatientID, 
				study_id AS StudyID, study_date_time AS StudyDate,
				accession_number AS AccessionNumber,
				study_instance_uid AS StudyInstanceUID, dbo.CONFIGURATION.Value + '\' + dbo.DICOM_FILES.rel_path AS file_path
FROM         dbo.DICOM_FILES CROSS JOIN
                      dbo.CONFIGURATION
WHERE     (dbo.CONFIGURATION.[Key] = 'root_dir')

GO


SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE VIEW [dbo].[CMOVE_SERIES_VIEW]
AS
SELECT DISTINCT 
                      patient_name AS PatientsName, patient_id AS PatientID, 
				study_id AS StudyID, study_date_time AS StudyDate,
				accession_number AS AccessionNumber,
				study_instance_uid AS StudyInstanceUID, modality AS Modality,
				series_number AS SeriesNumber,
				series_instance_uid AS SeriesInstanceUID, 
                      dbo.CONFIGURATION.Value + '\' + dbo.DICOM_FILES.rel_path AS file_path
FROM         dbo.DICOM_FILES CROSS JOIN
                      dbo.CONFIGURATION
WHERE     (dbo.CONFIGURATION.[Key] = 'root_dir')

GO


SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE VIEW [dbo].[CMOVE_IMAGE_VIEW]
AS
SELECT DISTINCT 
                      patient_name AS PatientsName, patient_id AS PatientID, 
				study_id AS StudyID, study_date_time AS StudyDate,
				accession_number AS AccessionNumber,
				study_instance_uid AS StudyInstanceUID, modality AS Modality,
				series_number AS SeriesNumber,
				series_instance_uid AS SeriesInstanceUID, 
				instance_number AS InstanceNumber,
                sop_class_uid AS SOPClassUID, sop_instance_uid AS SOPInstanceUID, 
                      dbo.CONFIGURATION.Value + '\' + dbo.DICOM_FILES.rel_path AS file_path
FROM         dbo.DICOM_FILES CROSS JOIN
                      dbo.CONFIGURATION
WHERE     (dbo.CONFIGURATION.[Key] = 'root_dir')

GO

SET ANSI_NULLS ON

GO

SET QUOTED_IDENTIFIER ON

GO

CREATE VIEW [dbo].[STORAGE_COMMIT_FAILED_INSTANCES]

AS

SELECT     dbo.STORAGE_COMMIT_REQUEST_INSTANCES.scm_id, dbo.STORAGE_COMMIT_REQUEST_INSTANCES.sop_class_uid, 

                      dbo.STORAGE_COMMIT_REQUEST_INSTANCES.sop_instance_uid

FROM         dbo.STORAGE_COMMIT_REQUEST_INSTANCES LEFT OUTER JOIN

                      dbo.DICOM_FILES ON dbo.STORAGE_COMMIT_REQUEST_INSTANCES.sop_class_uid = dbo.DICOM_FILES.sop_class_uid AND 

                      dbo.STORAGE_COMMIT_REQUEST_INSTANCES.sop_instance_uid = dbo.DICOM_FILES.sop_instance_uid

WHERE     (dbo.DICOM_FILES.file_id IS NULL)

GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO


SET ANSI_NULLS ON

GO

SET QUOTED_IDENTIFIER ON

GO

CREATE PROCEDURE [dbo].[SetLocalAE]

	-- Add the parameters for the stored procedure here

	@ae_title nvarchar(16), 

	@port int

AS

BEGIN

	SET NOCOUNT ON;

	update CONFIGURATION set [Value] = @ae_title WHERE ([Key] = 'ae_title')

	update CONFIGURATION set [Value] = @port WHERE ([Key] = 'port')

END

GO

SET ANSI_NULLS ON

GO

SET QUOTED_IDENTIFIER ON

GO


SET ANSI_NULLS ON

GO

SET QUOTED_IDENTIFIER ON

GO

CREATE PROCEDURE [dbo].[SetVerbose]

(

	@param1 nvarchar(50)

)

AS

	SET NOCOUNT OFF;

UPDATE    CONFIGURATION

SET              Value = @param1

WHERE     ([Key] = 'Verbose')

GO

SET ANSI_NULLS ON

GO

SET QUOTED_IDENTIFIER ON

GO

CREATE PROCEDURE [dbo].[SetDicomLog]

(

	@param1 nvarchar(50)

)

AS

	SET NOCOUNT OFF;

UPDATE    CONFIGURATION

SET              Value = @param1

WHERE     ([Key] = 'dicom_log')

GO

SET ANSI_NULLS ON

GO

SET QUOTED_IDENTIFIER ON

GO

CREATE PROCEDURE [dbo].[GetLocalAE]  @ae_title nvarchar(16) output,  @port int output

AS

SET NOCOUNT ON;

SELECT @ae_title = [Value] FROM CONFIGURATION WHERE ([Key] = 'ae_title')

SELECT @port = [Value] FROM CONFIGURATION WHERE ([Key] = 'port')

GO

SET ANSI_NULLS ON

GO

SET QUOTED_IDENTIFIER ON

GO



CREATE PROCEDURE [dbo].[GetFirstImage]
AS

BEGIN
	
	-- SET NOCOUNT ON added to prevent extra result sets from

	-- interfering with SELECT statements.

	SET NOCOUNT ON;

	SELECT 
		Top 1 [DICOM_FILES].[rel_path]
	FROM [DICOM_FILES]
	GROUP BY 
		[DICOM_FILES].[rel_path]
	
END

GO

SET ANSI_NULLS ON

GO

SET QUOTED_IDENTIFIER ON

GO



CREATE PROCEDURE [dbo].[GetFirstImageInSeriesPath]
	@SeriesUID nvarchar(MAX)
AS

BEGIN
	
	-- SET NOCOUNT ON added to prevent extra result sets from

	-- interfering with SELECT statements.

	SET NOCOUNT ON;

	SELECT 
		Top 1 [DICOM_FILES].[rel_path]
	FROM [DICOM_FILES]
	WHERE
		([DICOM_FILES].[series_instance_uid] = @SeriesUID)
	GROUP BY 
		[DICOM_FILES].[rel_path]

END

GO

SET ANSI_NULLS ON

GO

SET QUOTED_IDENTIFIER ON

GO




CREATE PROCEDURE [dbo].[GetFirstStudy]
AS

BEGIN
	
	-- SET NOCOUNT ON added to prevent extra result sets from

	-- interfering with SELECT statements.

	SET NOCOUNT ON;

	SELECT 
		TOP 1 [DICOM_FILES].[study_instance_uid]
	FROM [DICOM_FILES]
	GROUP BY 
		[DICOM_FILES].[study_instance_uid]
	
END

GO

SET ANSI_NULLS ON

GO

SET QUOTED_IDENTIFIER ON

GO



CREATE PROCEDURE [dbo].[GetImagePaths]
	@SeriesUID nvarchar(MAX)
AS

BEGIN
	
	-- SET NOCOUNT ON added to prevent extra result sets from

	-- interfering with SELECT statements.

	SET NOCOUNT ON;

	SELECT 
		[DICOM_FILES].[rel_path]
	FROM [DICOM_FILES]
	WHERE
		([DICOM_FILES].[series_instance_uid] = @SeriesUID)
	GROUP BY 
		[DICOM_FILES].[rel_path]

END

GO


SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[GetRootDir]

AS

SET NOCOUNT ON;

SELECT [Value] FROM CONFIGURATION WHERE ([Key] = 'root_dir')

GO

SET ANSI_NULLS ON

GO

SET QUOTED_IDENTIFIER ON

GO



CREATE PROCEDURE [dbo].[GetSeries]
	@StudyUID nvarchar(MAX)
AS

BEGIN
	
	-- SET NOCOUNT ON added to prevent extra result sets from

	-- interfering with SELECT statements.

	SET NOCOUNT ON;

	SELECT 
		[DICOM_FILES].[series_instance_uid], [DICOM_FILES].[study_instance_uid]
	FROM [DICOM_FILES]
	WHERE
		([DICOM_FILES].[study_instance_uid] = @StudyUID)
	GROUP BY 
		[DICOM_FILES].[series_instance_uid], [DICOM_FILES].[study_instance_uid]
	
END

GO

SET ANSI_NULLS ON

GO

SET QUOTED_IDENTIFIER ON

GO



CREATE PROCEDURE [dbo].[GetStudies]
	@PatientID nvarchar(MAX),
	@PatientName nvarchar(MAX)
AS

BEGIN
	
	-- SET NOCOUNT ON added to prevent extra result sets from

	-- interfering with SELECT statements.

	SET NOCOUNT ON;

	SELECT 
		[DICOM_FILES].[study_instance_uid], 
		Count(*) AS NumberOfSeries 
	FROM [DICOM_FILES]
	WHERE
		([DICOM_FILES].[patient_id] like @PatientID AND [DICOM_FILES].[patient_name] like @PatientName)
	GROUP BY 
		[DICOM_FILES].[study_instance_uid]
	
END

GO


SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[GetWebVirtualDir]

AS

SET NOCOUNT ON;

SELECT [Value] FROM CONFIGURATION WHERE ([Key] = 'web_virtual_dir')

GO



SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[GetWebDir]

AS

SET NOCOUNT ON;

SELECT [Value] FROM CONFIGURATION WHERE ([Key] = 'web_dir')

GO




CREATE PROCEDURE [dbo].[CleanOldSCMs] 
AS
BEGIN
	SET NOCOUNT ON;
	declare @cleantime datetime
	select @cleantime = (GETDATE() - (SELECT     CAST(Value AS float) AS Expr1
                            FROM          dbo.CONFIGURATION
                            WHERE      ([Key] = 'cleanup_seconds')) / 60 / 60 / 24)
	delete from storage_commit_request
	where (creation_time < @cleantime AND update_time < @cleantime) OR
		  (creation_time < @cleantime AND (update_time IS NULL))
END
GO



ALTER TABLE [dbo].[STORAGE_COMMIT_REQUEST_INSTANCES]  WITH CHECK ADD  CONSTRAINT [FK_STORAGE_COMMIT_REQUEST_INSTANCES_STORGAE_COMMIT_REQUEST] FOREIGN KEY([scm_id])
REFERENCES [dbo].[STORAGE_COMMIT_REQUEST] ([scm_id])
ON DELETE CASCADE
GO
ALTER TABLE [dbo].[STORAGE_COMMIT_REQUEST_INSTANCES] CHECK CONSTRAINT [FK_STORAGE_COMMIT_REQUEST_INSTANCES_STORGAE_COMMIT_REQUEST]
GO



/****** Object:  Table [dbo].[SERVICE_REQUEST]    Script Date: 17/08/2014 08:24:46 ******/
SET ANSI_NULLS ON
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE TABLE [dbo].[SERVICE_REQUEST](
	[id] [int] IDENTITY(1,1) NOT NULL,
	[accession_number] [nvarchar](50) NULL,
	[referring_physicians_names] [nvarchar](400) NULL,
	[requesting_physician] [nvarchar](400) NULL,
	[requesting_service] [nvarchar](100) NULL,
 CONSTRAINT [PK_SERVICE_REQUEST] PRIMARY KEY CLUSTERED 
(
	[id] ASC
)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY]
) ON [PRIMARY]

GO


/****** Object:  Table [dbo].[REQUESTED_PROCEDURE_CODES]    Script Date: 14/08/2014 11:12:34 ******/
SET ANSI_NULLS ON
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE TABLE [dbo].[REQUESTED_PROCEDURE_CODES](
	[ReqProcCode] [int] IDENTITY(1,1) NOT NULL,
	[RequestedProcedureDescription] [nvarchar](64) NOT NULL,
 CONSTRAINT [PK_REQUESTED_PROCEDURE_CODES] PRIMARY KEY CLUSTERED 
(
	[ReqProcCode] ASC
)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY]
) ON [PRIMARY]

GO


INSERT INTO [dbo].[REQUESTED_PROCEDURE_CODES] ([RequestedProcedureDescription]) VALUES ('Physical examination')
INSERT INTO [dbo].[REQUESTED_PROCEDURE_CODES] ([RequestedProcedureDescription]) VALUES ('Tests')
INSERT INTO [dbo].[REQUESTED_PROCEDURE_CODES] ([RequestedProcedureDescription]) VALUES ('X-rays')
INSERT INTO [dbo].[REQUESTED_PROCEDURE_CODES] ([RequestedProcedureDescription]) VALUES ('Scans')
INSERT INTO [dbo].[REQUESTED_PROCEDURE_CODES] ([RequestedProcedureDescription]) VALUES ('Therapy')
INSERT INTO [dbo].[REQUESTED_PROCEDURE_CODES] ([RequestedProcedureDescription]) VALUES ('Rehabilitation procedure')
INSERT INTO [dbo].[REQUESTED_PROCEDURE_CODES] ([RequestedProcedureDescription]) VALUES ('Cosmetic procedure')
GO



/****** Object:  Table [dbo].[REQUESTED_PROCEDURE]    Script Date: 26/10/2015 11:08:08 ******/
SET ANSI_NULLS ON
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE TABLE [dbo].[REQUESTED_PROCEDURE](
	[id] [int] IDENTITY(1,1) NOT NULL,
	[study_instance_uid] [nvarchar](64) NULL,
	[req_proc_desc] [nvarchar](64) NULL,
	[req_proc_id] [nvarchar](50) NULL,
	[service_request_id] [int] NULL,
	[patient_internal_id] [int] NULL,
	[RequestedProcedureCodeSequence] [int] NULL,
 CONSTRAINT [PK_REQUESTED_PROCEDURE] PRIMARY KEY CLUSTERED 
(
	[id] ASC
)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY]
) ON [PRIMARY]

GO


/****** Object:  Table [dbo].[SCHED_PROC_STEP]    Script Date: 26/10/2015 11:06:42 ******/
SET ANSI_NULLS ON
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE TABLE [dbo].[SCHED_PROC_STEP](
	[id] [int] IDENTITY(1,1) NOT NULL,
	[modality] [nvarchar](16) NULL,
	[scheduled_station_ae_title] [nvarchar](16) NULL,
	[start_date_time] [datetime] NULL,
	[end_date_time] [datetime] NULL,
	[perf_phys_name] [nvarchar](250) NULL,
	[sched_proc_step_desc] [nvarchar](64) NULL,
	[sched_proc_step_id] [nvarchar](16) NULL,
	[req_proc_id] [int] NOT NULL,
 CONSTRAINT [PK_SCHED_PROC_STEP] PRIMARY KEY CLUSTERED 
(
	[id] ASC
)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY]
) ON [PRIMARY]

GO

ALTER TABLE [dbo].[SCHED_PROC_STEP] ADD  CONSTRAINT [DF_SCHED_PROC_STEP_start_date_time]  DEFAULT (getdate()) FOR [start_date_time]
GO


SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE TABLE [dbo].[DICOM_CODES](
	[id] [int] IDENTITY(1,1) NOT NULL,
	[CodeValue] [nvarchar](50) NULL,
	[CodingSchemeDesignator] [nvarchar](50) NULL,
	[CodingSchemeVersion] [nvarchar](50) NULL,
	[CodeMeaning] [nvarchar](50) NULL,
 CONSTRAINT [PK_DICOM_CODES] PRIMARY KEY CLUSTERED 
(
	[id] ASC
)WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]
GO


/****** Object:  View [dbo].[CFIND_MWL_VIEW]    Script Date: 17/08/2014 08:28:01 ******/
SET ANSI_NULLS ON
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE TABLE [MPPS] (
	[id] [int] IDENTITY(1,1) NOT NULL PRIMARY KEY,
	[sop_instance_uid] [nvarchar](64) NOT NULL,
	[patient_name] [nvarchar](64) NULL,
	[patient_id] [nvarchar](64) NULL,
	[patient_sex] [nchar](1) NULL,
	[patient_birth_date] [datetime] NULL,
	[pps_id] [nvarchar](16) NOT NULL,
	[performed_station_ae_title] [nvarchar](16) NULL,
	[performed_station_name] [nvarchar](16) NULL,
	[performed_location] [nvarchar](16) NULL,
	[pps_start_datetime] [datetime] NULL,
	[pps_status] [nvarchar](16) NOT NULL,
	[pps_desc] [nvarchar](64) NULL,
	[performed_procedure_type_desc] [nvarchar](64) NULL,
	[pps_end_datetime] [datetime] NULL,
	[modality] [nvarchar](16) NULL,
	[study_id] [nvarchar](16) NULL,
	[rel_path] [nvarchar](400) NOT NULL,
)
GO

CREATE TABLE [MPPS_SPS](
	[id] [int] IDENTITY(1,1) NOT NULL PRIMARY KEY,
	[mpps_id] [int] NOT NULL FOREIGN KEY REFERENCES MPPS(id) ON DELETE CASCADE,
	[study_instance_uid] [nvarchar](64) NOT NULL,
	[accession_number] [nvarchar](16) NULL,
	[requested_procedure_id] [nvarchar](50) NULL,
	[requested_procedure_desc] [nvarchar](400) NULL,
	[sps_id] [nvarchar](16) NULL,
	[sps_desc] [nvarchar](400) NULL,
)
GO

CREATE VIEW [dbo].[AWAITING_REQUESTED_PROCEDURES_VIEW]
AS
SELECT dbo.REQUESTED_PROCEDURE.id, dbo.REQUESTED_PROCEDURE.study_instance_uid, dbo.REQUESTED_PROCEDURE.req_proc_desc, dbo.REQUESTED_PROCEDURE.req_proc_id, dbo.REQUESTED_PROCEDURE.service_request_id, dbo.REQUESTED_PROCEDURE.patient_internal_id, 
         dbo.REQUESTED_PROCEDURE.RequestedProcedureCodeSequence
FROM  dbo.REQUESTED_PROCEDURE LEFT OUTER JOIN
         dbo.MPPS_SPS ON dbo.REQUESTED_PROCEDURE.study_instance_uid = dbo.MPPS_SPS.study_instance_uid
WHERE (dbo.MPPS_SPS.study_instance_uid IS NULL)
GO

-- Change the MWL view to take only open procedures
Create VIEW [dbo].[CFIND_MWL_VIEW]
AS
SELECT dbo.AWAITING_REQUESTED_PROCEDURES_VIEW.study_instance_uid AS StudyInstanceUID, dbo.AWAITING_REQUESTED_PROCEDURES_VIEW.req_proc_desc AS RequestedProcedureDescription, dbo.AWAITING_REQUESTED_PROCEDURES_VIEW.id AS ScheduledProcedureStepSequence, 
         dbo.AWAITING_REQUESTED_PROCEDURES_VIEW.req_proc_id AS RequestedProcedureID, dbo.SERVICE_REQUEST.accession_number AS AccessionNumber, dbo.SERVICE_REQUEST.referring_physicians_names AS ReferringPhysicianName, dbo.SERVICE_REQUEST.requesting_physician AS RequestingPhysician, dbo.N_PATIENT.patient_id AS PatientID, 
         dbo.N_PATIENT.patient_name AS PatientName, CONVERT(char(8), dbo.N_PATIENT.birth_date, 112) AS PatientBirthDate, dbo.N_PATIENT.sex AS PatientSex, NULL AS PatientWeight, NULL AS MedicalAlerts, NULL AS Allergies, NULL AS PregnancyStatus, NULL AS RequestedContrastAgent, NULL AS AdmissionID, NULL AS SpecialNeeds, NULL 
         AS CurrentPatientLocation, NULL AS PatientState, NULL AS RequestedProcedurePriority, NULL AS PatientTransportArrangements, NULL AS ConfidentialityConstraintOnPatientDataDescription, dbo.SERVICE_REQUEST.requesting_service
FROM  dbo.AWAITING_REQUESTED_PROCEDURES_VIEW INNER JOIN
         dbo.SERVICE_REQUEST ON dbo.AWAITING_REQUESTED_PROCEDURES_VIEW.service_request_id = dbo.SERVICE_REQUEST.id INNER JOIN
         dbo.N_PATIENT ON dbo.AWAITING_REQUESTED_PROCEDURES_VIEW.patient_internal_id = dbo.N_PATIENT.id

GO

CREATE VIEW [dbo].[MPPS_SPS_JOINED_VIEW]
AS
SELECT dbo.MPPS.*, dbo.MPPS_SPS.study_instance_uid, dbo.MPPS_SPS.accession_number, dbo.MPPS_SPS.requested_procedure_id, dbo.MPPS_SPS.requested_procedure_desc, dbo.MPPS_SPS.sps_desc, 
                  dbo.MPPS_SPS.sps_id
FROM     dbo.MPPS INNER JOIN
                  dbo.MPPS_SPS ON dbo.MPPS.id = dbo.MPPS_SPS.mpps_id
GO

create view WORKLIST_VIEW
as
SELECT dbo.N_PATIENT.id AS patId, dbo.N_PATIENT.patient_id, dbo.N_PATIENT.patient_name, dbo.N_PATIENT.birth_date, dbo.N_PATIENT.sex, dbo.N_PATIENT.phone_number, dbo.N_PATIENT.address, 
                  dbo.REQUESTED_PROCEDURE.id AS reqProcId, dbo.REQUESTED_PROCEDURE.study_instance_uid, dbo.REQUESTED_PROCEDURE.req_proc_desc, dbo.REQUESTED_PROCEDURE.req_proc_id AS RPID, 
                  dbo.SERVICE_REQUEST.id AS servReqId, dbo.SERVICE_REQUEST.accession_number, dbo.SERVICE_REQUEST.referring_physicians_names, dbo.SERVICE_REQUEST.requesting_physician, 
                  dbo.SERVICE_REQUEST.requesting_service, dbo.SCHED_PROC_STEP.id AS shedStepId, dbo.SCHED_PROC_STEP.modality, dbo.SCHED_PROC_STEP.scheduled_station_ae_title, 
                  dbo.SCHED_PROC_STEP.start_date_time, dbo.SCHED_PROC_STEP.perf_phys_name, dbo.SCHED_PROC_STEP.sched_proc_step_desc, dbo.SCHED_PROC_STEP.sched_proc_step_id, 
                  dbo.MPPS_SPS_JOINED_VIEW.pps_start_datetime, dbo.MPPS_SPS_JOINED_VIEW.pps_end_datetime, dbo.MPPS_SPS_JOINED_VIEW.performed_station_ae_title, dbo.MPPS_SPS_JOINED_VIEW.pps_status, 
                  dbo.MPPS_SPS_JOINED_VIEW.pps_desc
FROM     dbo.REQUESTED_PROCEDURE INNER JOIN
                  dbo.N_PATIENT ON dbo.REQUESTED_PROCEDURE.patient_internal_id = dbo.N_PATIENT.id INNER JOIN
                  dbo.SERVICE_REQUEST ON dbo.REQUESTED_PROCEDURE.service_request_id = dbo.SERVICE_REQUEST.id INNER JOIN
                  dbo.SCHED_PROC_STEP ON dbo.REQUESTED_PROCEDURE.id = dbo.SCHED_PROC_STEP.req_proc_id LEFT OUTER JOIN
                  dbo.MPPS_SPS_JOINED_VIEW ON dbo.SCHED_PROC_STEP.sched_proc_step_id = dbo.MPPS_SPS_JOINED_VIEW.sps_id
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE VIEW [dbo].[CFIND_ScheduledProcedureStepSequence_VIEW]
AS
SELECT        
	id, 
	modality, 
	scheduled_station_ae_title AS ScheduledStationAETitle, 
	CONVERT(char(8), start_date_time, 112) AS ScheduledProcedureStepStartDate, 
    perf_phys_name AS ScheduledPerformingPhysicianName, 
    req_proc_id AS ScheduledProcedureStepSequence, 
    REPLACE(CONVERT(char(8), start_date_time, 108), ':', '') AS ScheduledProcedureStepStartTime, 
    sched_proc_step_desc AS ScheduledProcedureStepDescription, 
    sched_proc_step_id AS ScheduledProcedureStepID, 
    NULL AS ScheduledPerformingPhysiciansName, 
    NULL AS ScheduledStationName, 
    NULL AS ScheduledProcedureStepLocation, 
    NULL AS PreMedication,
	start_date_time AS ScheduledProcedureStepStartDateTime, 
    CONVERT(char(8), end_date_time, 112) AS ScheduledProcedureStepEndDate, 
    REPLACE(CONVERT(char(8), end_date_time, 108), ':', '')AS ScheduledProcedureStepEndTime, 
    end_date_time AS DateTime
FROM            
	dbo.SCHED_PROC_STEP
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE VIEW [dbo].[CONFIGURATION_VIEW]
AS
SELECT      
	[Key] AS confkey,
	[Value] AS confvalue,
	[Description] AS confdescription
FROM            
	dbo.CONFIGURATION
GO


ALTER TABLE [dbo].[REQUESTED_PROCEDURE]  WITH CHECK ADD  CONSTRAINT [FK_REQUESTED_PROCEDURE_N_PATIENT] FOREIGN KEY([patient_internal_id])
REFERENCES [dbo].[N_PATIENT] ([id])
GO
ALTER TABLE [dbo].[REQUESTED_PROCEDURE] CHECK CONSTRAINT [FK_REQUESTED_PROCEDURE_N_PATIENT]
GO
ALTER TABLE [dbo].[REQUESTED_PROCEDURE]  WITH CHECK ADD  CONSTRAINT [FK_REQUESTED_PROCEDURE_SERVICE_REQUEST] FOREIGN KEY([service_request_id])
REFERENCES [dbo].[SERVICE_REQUEST] ([id])
GO
ALTER TABLE [dbo].[REQUESTED_PROCEDURE] CHECK CONSTRAINT [FK_REQUESTED_PROCEDURE_SERVICE_REQUEST]
GO
ALTER TABLE [dbo].[SCHED_PROC_STEP]  WITH CHECK ADD  CONSTRAINT [FK_SCHED_PROC_STEP_REQUESTED_PROCEDURE] FOREIGN KEY([req_proc_id])
REFERENCES [dbo].[REQUESTED_PROCEDURE] ([id])
GO
ALTER TABLE [dbo].[SCHED_PROC_STEP] CHECK CONSTRAINT [FK_SCHED_PROC_STEP_REQUESTED_PROCEDURE]
GO


/****** Insert data into the CONFIGURATION table ******/
INSERT INTO [dbo].[CONFIGURATION] ([Key],[Value],[Description]) VALUES ('ae_title', 'DSRSVC', 'Local ae title');
INSERT INTO [dbo].[CONFIGURATION] ([Key],[Value],[Description]) VALUES ('port', '104', 'Listener port');
INSERT INTO [dbo].[CONFIGURATION] ([Key],[Value],[Description]) VALUES ('retry_seconds', '180', 'The number of seconds to wait before retrying a failed task');
INSERT INTO [dbo].[CONFIGURATION] ([Key],[Value],[Description]) VALUES ('root_dir', '\DSRSVC', 'Working file system directory');
INSERT INTO [dbo].[CONFIGURATION] ([Key],[Value],[Description]) VALUES ('scm_max_retry', '3', 'The maximum number of retries for sending back commit responses');
INSERT INTO [dbo].[CONFIGURATION] ([Key],[Value],[Description]) VALUES ('dicom_log', '0', 'When this value is 1, detailed logging is on');
INSERT INTO  [dbo].[CONFIGURATION]  ([Key],[Value],[Description]) VALUES ('scm_on_same_assoc', '1', 'When this value is 1, storage_commit is performed on a new association');



