SET SESSION TRANSACTION ISOLATION LEVEL SERIALIZABLE;
SET SESSION sql_mode = 'ANSI';
CREATE DATABASE IF NOT EXISTS RZ_DICOM_HL7;

Use RZ_DICOM_HL7;
SET GLOBAL TRANSACTION ISOLATION LEVEL SERIALIZABLE;
SET GLOBAL sql_mode = 'ANSI';

CREATE TABLE MESSAGES(
	message_id INT NOT NULL AUTO_INCREMENT,
	message_date_time DATE NULL,
	message_type NVARCHAR(400) NULL,
	message_control_id NVARCHAR(400) NULL,
	sending_application NVARCHAR(400) NULL,
	sending_facility NVARCHAR(400) NULL,
	receiving_application NVARCHAR(400) NULL,
	receiving_facility NVARCHAR(400) NULL,
    PRIMARY KEY (message_id));

CREATE TABLE QUERY_DEFINITION (
     query_def_id   int NOT NULL AUTO_INCREMENT,
     query_message_id   int  NULL,
	 query_date_time   datetime  NULL,
	 query_format_code   nvarchar (400) NULL,
	 query_priority   nvarchar (400) NULL,
	 query_id   nvarchar (400) NULL,
	 quantity_limited_restrict   nvarchar (400) NULL,
	 who_subject_filter   nvarchar (400) NULL,
	 what_subject_filter   nvarchar (400) NULL,
PRIMARY KEY (query_def_id),
FOREIGN KEY FK_QUERY_DEFINITION_MESSAGES (query_message_id)
REFERENCES MESSAGES(message_id)
ON UPDATE CASCADE
ON DELETE CASCADE
); 

CREATE TABLE   N_PATIENT (
	 id   int    NOT NULL AUTO_INCREMENT,
	 patient_id   nvarchar (64) NULL,
	 patient_message_id   int  NULL,
	 patient_name   nvarchar (400) NULL,
	 phone_number   nvarchar (400) NULL,
	 birth_date   datetime  NULL,
	 sex   char (1) NULL,
	 address   nvarchar (400) NULL,
	 ssn   numeric (18, 0) NULL,
PRIMARY KEY (id),
FOREIGN KEY FK_N_PATIENT_MESSAGES (patient_message_id)
REFERENCES MESSAGES(message_id)
ON UPDATE CASCADE
ON DELETE CASCADE);

CREATE TABLE   REPORTS (
	 report_id   int    NOT NULL AUTO_INCREMENT,
	 report_patient_id   int  NULL,
	 set_id   int  NULL,
	 placer_order_no   nvarchar (400) NULL,
	 fillers_order_no   nvarchar (400) NULL,
	 universal_service_id   nvarchar (400) NULL,
	 observation_date_time   datetime  NULL,
	 ordering_provider   nvarchar (400) NULL,
	 result_status   nvarchar (400) NULL,
	 prin_result_interpreter   nvarchar (400) NULL,
 PRIMARY KEY (report_id),
 FOREIGN KEY FK_REPORTS_N_PATIENT (report_patient_id)
REFERENCES N_PATIENT(id)
ON UPDATE CASCADE
ON DELETE CASCADE);

CREATE TABLE   REPORT_ITEMS (
	 report_item_id   int    NOT NULL AUTO_INCREMENT,
	 reportitem_report_id   int  NULL,
	 set_id   int  NULL,
	 value_type   nvarchar (400) NULL,
	 observation_identifer   nvarchar (400) NULL,
	 observation_results   nvarchar (400) NULL,
	 units   nvarchar (400) NULL,
	 reference_range   nvarchar (400) NULL,
	 abnormal_flags   nvarchar (400) NULL,
	 observ_result_status   nvarchar (400) NULL,
 PRIMARY KEY (report_item_id),
 FOREIGN KEY FK_REPORT_ITEMS_REPORTS (reportitem_report_id)
REFERENCES REPORTS(report_id)
ON UPDATE CASCADE
ON DELETE CASCADE); 

CREATE TABLE   HL7_QUEUE (
	 que_id   int    NOT NULL AUTO_INCREMENT,
	 que_status   int  NOT NULL,
	 que_creation_date   datetime  NULL DEFAULT NOW(),
	 que_event_type   nchar (10) NULL,
	 que_start_table   nvarchar (100) NULL,
	 que_start_field   nvarchar (100) NULL,
	 que_start_value   int  NULL,
	 que_update_date   datetime  NULL,
	 que_data_type   nchar (10) NULL,
	 que_retry_count   int  NULL DEFAULT 0,
	 que_last_error_code   int  NULL,
	 que_last_error_message   nvarchar (400) NULL,
	 que_app_name   nvarchar (100) NULL,
	 que_app_facility   nvarchar (100) NULL,
 PRIMARY KEY (que_id ASC)); 
 
 
DELIMITER //
CREATE TRIGGER   RZ_MESSAGES_message_id  
   AFTER INSERT
   ON  MESSAGES
   FOR EACH ROW
BEGIN
IF NEW.message_type = 'QRY^A19' THEN
       INSERT INTO   HL7_QUEUE 
           ( que_status 
           , que_creation_date 
           , que_event_type 
           , que_start_table 
           , que_start_field 
           , que_start_value 
           , que_update_date 
           , que_data_type 
           , que_retry_count 
           , que_last_error_code 
           , que_last_error_message
           , que_app_name
		   , que_app_facility)
     VALUES
           (2
           ,NOW()
           ,'QRYA19'
           ,'MESSAGES'
           ,'message_id'
           ,NEW.message_id
           ,NOW()
           ,'0'
           ,0
           ,0
           ,''
           ,''
           ,'');
ELSEIF (NEW.message_type = 'ORU^R01') THEN
       INSERT INTO   HL7_QUEUE 
           ( que_status 
           , que_creation_date 
           , que_event_type 
           , que_start_table 
           , que_start_field 
           , que_start_value 
           , que_update_date 
           , que_data_type 
           , que_retry_count 
           , que_last_error_code 
           , que_last_error_message 
           , que_app_name
		   , que_app_facility)
     VALUES
           (2
           ,NOW()
           ,'ORUR01'
           ,'MESSAGES'
           ,'message_id'
           ,NEW.message_id
           ,NOW()
           ,'0'
           ,0
           ,0
           ,''
           ,''
           ,'');
END IF;
END; //
DELIMITER ;


CREATE TABLE   STORAGE_COMMIT_REQUEST (
	 scm_id   int    NOT NULL AUTO_INCREMENT,
	 creation_time   datetime  NOT NULL DEFAULT NOW(),
	 update_time   datetime  NULL,
	 scm_source_ae   nvarchar (64) NOT NULL,
	 scm_transaction_uid   nvarchar (64) NOT NULL,
	 tasks_queued   int  NULL,
	 response_retry_count   int  NOT NULL DEFAULT 0,
	 response_status   int  NULL,
	 scm_kind   int  NOT NULL DEFAULT 0,
	 task_id   int  NULL,
	PRIMARY KEY (scm_id),
    CONSTRAINT UNIQUE INDEX IX_STORGAE_COMMIT_REQUEST (scm_transaction_uid  ASC));

CREATE TABLE   CONFIGURATION (
	 `Key`   nvarchar (50) NOT NULL,
	 `Value`   nvarchar (50) NOT NULL,
	 Description   nvarchar (400) NULL,
 PRIMARY KEY (`Key`));

CREATE TABLE   DICOM_APPLICATIONS (
	 id   int    NOT NULL AUTO_INCREMENT,
	 alias   nvarchar (400) NULL,
	 ae_title   nvarchar (64) NOT NULL,
	 host   nvarchar (64) NULL,
	 port   int  NULL,
     PRIMARY KEY (id),
     CONSTRAINT UNIQUE INDEX IX_DICOM_APPLICATIONS_AE_TITLE (ae_title  ASC)); 

CREATE TABLE   STORAGE_COMMIT_REQUEST_INSTANCES (
	 scm_id   int  NOT NULL,
	 sop_class_uid   nvarchar (64) NOT NULL,
	 sop_instance_uid   nvarchar (64) NOT NULL,
	 status   int  NOT NULL DEFAULT 2,
	 failure_reason   int  NOT NULL DEFAULT 0,
 PRIMARY KEY (scm_id  ASC, sop_class_uid  ASC, sop_instance_uid  ASC),
 CONSTRAINT UNIQUE INDEX IX_STORAGE_COMMIT_REQUEST_INSTANCES_1 (sop_class_uid  ASC, sop_instance_uid  ASC),
 FOREIGN KEY FK_STORAGE_COMMIT_REQUEST_INSTANCES_STORGAE_COMMIT_REQUEST (scm_id)
REFERENCES STORAGE_COMMIT_REQUEST (scm_id)
ON UPDATE CASCADE
ON DELETE CASCADE);

CREATE TABLE   DICOM_FILES (
	 file_id   int    NOT NULL AUTO_INCREMENT,
	 creation_time   datetime  NOT NULL DEFAULT NOW(),
	 update_time   datetime  NULL,
	 patient_name   nvarchar (64) NOT NULL,
	 patient_id   nvarchar (64) NOT NULL,
	 patient_sex   nchar (1) NULL,
	 patient_birth_date   datetime  NULL,
	 study_instance_uid   nvarchar (64) NOT NULL,
	 study_id   nvarchar (16) NULL,
	 study_date_time   datetime  NULL,
	 accession_number   nvarchar (16) NULL,
	 modality   nvarchar (16) NULL,
	 series_number   nvarchar (16) NULL,
	 series_instance_uid   nvarchar (64) NOT NULL,
	 instance_number   nvarchar (16) NULL,
	 sop_class_uid   nvarchar (64) NOT NULL,
	 sop_instance_uid   nvarchar (64) NOT NULL,
	 source_ae_title   nvarchar (64) NULL,
	 rel_path   nvarchar (400) NOT NULL,
	 task_id   int  NULL,
 PRIMARY KEY (file_id  ASC),
 CONSTRAINT UNIQUE INDEX IX_DICOM_FILES_1 (patient_name  ASC,
	 patient_id  ASC,
	 study_instance_uid  ASC,
	 series_instance_uid  ASC,
	 sop_class_uid  ASC,
	 sop_instance_uid  ASC,
	 task_id  ASC),
  INDEX IX_DICOM_FILES_2 (sop_class_uid  ASC, sop_instance_uid  ASC)); 

CREATE TABLE   SERVICE_REQUEST (
	 id   int    NOT NULL AUTO_INCREMENT,
	 accession_number   nvarchar (50) NULL,
	 referring_physicians_names   nvarchar (400) NULL,
	 requesting_physician   nvarchar (400) NULL,
	 requesting_service   nvarchar (100) NULL,
 PRIMARY KEY (id  ASC));

CREATE TABLE REQUESTED_PROCEDURE_CODES (
    ReqProcCode INT  NOT NULL AUTO_INCREMENT,
    RequestedProcedureDescription NVARCHAR(64) NOT NULL,
    PRIMARY KEY (ReqProcCode ASC)
);

CREATE TABLE REQUESTED_PROCEDURE (
    id INT  NOT NULL AUTO_INCREMENT,
    study_instance_uid NVARCHAR(64) NULL,
    req_proc_desc NVARCHAR(64) NULL,
    req_proc_id NVARCHAR(50) NULL,
    service_request_id INT NULL,
    patient_internal_id INT NULL,
    RequestedProcedureCodeSequence INT NULL,
    PRIMARY KEY (id ASC),
    FOREIGN KEY FK_REQUESTED_PROCEDURE_N_PATIENT (patient_internal_id)
REFERENCES N_PATIENT(id)
ON UPDATE CASCADE
ON DELETE CASCADE,
    FOREIGN KEY FK_REQUESTED_PROCEDURE_SERVICE_REQUEST (service_request_id)
REFERENCES SERVICE_REQUEST(id)

);

CREATE TABLE SCHED_PROC_STEP (
    id INT  NOT NULL AUTO_INCREMENT,
    modality NVARCHAR(16) NULL,
    scheduled_station_ae_title NVARCHAR(16) NULL,
    start_date_time DATETIME NULL DEFAULT NOW(),
    end_date_time DATETIME NULL,
    perf_phys_name NVARCHAR(250) NULL,
    sched_proc_step_desc NVARCHAR(64) NULL,
    sched_proc_step_id NVARCHAR(16) NULL,
    req_proc_id INT NOT NULL,
    PRIMARY KEY (id ASC),
    FOREIGN KEY FK_SCHED_PROC_STEP_REQUESTED_PROCEDURE (req_proc_id)
        REFERENCES REQUESTED_PROCEDURE (id)
        ON UPDATE CASCADE ON DELETE CASCADE
);

CREATE TABLE   DICOM_CODES (
	 id   int    NOT NULL AUTO_INCREMENT,
	 CodeValue   nvarchar (50) NULL,
	 CodingSchemeDesignator   nvarchar (50) NULL,
	 CodingSchemeVersion   nvarchar (50) NULL,
	 CodeMeaning   nvarchar (50) NULL,
 PRIMARY KEY (id  ASC));

CREATE VIEW   STORAGE_COMMIT_SUCCEEDED_INSTANCES 
AS
SELECT      STORAGE_COMMIT_REQUEST_INSTANCES.scm_id,  STORAGE_COMMIT_REQUEST_INSTANCES.sop_class_uid, 
                       STORAGE_COMMIT_REQUEST_INSTANCES.sop_instance_uid
FROM          STORAGE_COMMIT_REQUEST_INSTANCES INNER JOIN
                       DICOM_FILES ON  STORAGE_COMMIT_REQUEST_INSTANCES.sop_class_uid =  DICOM_FILES.sop_class_uid AND 
                       STORAGE_COMMIT_REQUEST_INSTANCES.sop_instance_uid =  DICOM_FILES.sop_instance_uid;

CREATE VIEW CONFIGURATION_VIEW
AS
SELECT      
	`Key` AS confkey,
	`Value` AS confvalue,
	Description AS confdescription
FROM            
	CONFIGURATION;
    
CREATE VIEW   CFIND_IMAGE_VIEW 
AS
SELECT DISTINCT 
                patient_name AS PatientsName, patient_id AS PatientID, 
				study_id AS StudyID, study_date_time AS StudyDate,
				accession_number AS AccessionNumber,
				study_instance_uid AS StudyInstanceUID, modality AS Modality,
				series_number AS SeriesNumber,
				series_instance_uid AS SeriesInstanceUID, 
				instance_number AS InstanceNumber,
                sop_class_uid AS SOPClassUID, sop_instance_uid AS SOPInstanceUID
FROM          DICOM_FILES;

CREATE VIEW   CFIND_SERIES_VIEW 
AS
SELECT DISTINCT 
                patient_name AS PatientsName, patient_id AS PatientID, 
				study_id AS StudyID, study_date_time AS StudyDate,
				accession_number AS AccessionNumber,
				study_instance_uid AS StudyInstanceUID, modality AS Modality,
				series_number AS SeriesNumber,
				series_instance_uid AS SeriesInstanceUID
FROM          DICOM_FILES;


CREATE VIEW CFIND_STUDY_VIEW
AS
SELECT DISTINCT patient_name AS PatientName, patient_id AS PatientID, study_id AS StudyID,
                (SELECT CAST(DICOM_FILES.study_date_time AS DATE) AS Expr1) AS StudyDate,
                (SELECT CAST(DICOM_FILES.study_date_time AS TIME) AS Expr1) AS StudyTime, 
                accession_number AS AccessionNumber, study_instance_uid AS StudyInstanceUID, patient_sex AS PatientSex, patient_birth_date AS PatientBirthDate,
                (SELECT CAST(COUNT(SeriesInstanceUID) AS char(12)) FROM CFIND_SERIES_VIEW WHERE (StudyInstanceUID = DICOM_FILES.study_instance_uid)) AS NumberOfStudyRelatedSeries,
                (SELECT CAST(COUNT(SOPInstanceUID) AS char(12)) FROM    CFIND_IMAGE_VIEW WHERE (StudyInstanceUID = DICOM_FILES.study_instance_uid)) AS NumberOfStudyRelatedInstances,
                (SELECT DISTINCT CONCAT(Modality,'\\') AS 'data()' FROM    CFIND_IMAGE_VIEW AS CFIND_IMAGE_VIEW_1 WHERE  (StudyInstanceUID = DICOM_FILES.study_instance_uid) ) AS ModalitiesInStudy
FROM    DICOM_FILES;


CREATE VIEW   CFIND_PATIENT_VIEW 
AS
SELECT DISTINCT 
    patient_name AS PatientName, 
	patient_id AS PatientID, 
	patient_sex AS PatientSex, 
	patient_birth_date AS PatientBirthDate
FROM          DICOM_FILES;

CREATE VIEW   CMOVE_PATIENT_VIEW 
AS
SELECT DISTINCT 
              DICOM_FILES.patient_name AS PatientsName,  DICOM_FILES.patient_id AS PatientID, 
              CONCAT(CONFIGURATION.`Value`,'\\', DICOM_FILES.rel_path) AS file_path
FROM          DICOM_FILES CROSS JOIN
                       CONFIGURATION
WHERE     ( CONFIGURATION.`Key`  = 'root_dir');

CREATE VIEW   CMOVE_STUDY_VIEW 
AS
SELECT DISTINCT 
                patient_name AS PatientsName, patient_id AS PatientID, 
				study_id AS StudyID, study_date_time AS StudyDate,
				accession_number AS AccessionNumber,
				study_instance_uid AS StudyInstanceUID,  
                CONCAT(CONFIGURATION.`Value`,'\\', DICOM_FILES.rel_path) AS file_path
FROM          DICOM_FILES CROSS JOIN
                       CONFIGURATION
WHERE     ( CONFIGURATION.`Key`  = 'root_dir');

CREATE VIEW   CMOVE_SERIES_VIEW 
AS
SELECT DISTINCT 
                      patient_name AS PatientsName, patient_id AS PatientID, 
				study_id AS StudyID, study_date_time AS StudyDate,
				accession_number AS AccessionNumber,
				study_instance_uid AS StudyInstanceUID, modality AS Modality,
				series_number AS SeriesNumber,
				series_instance_uid AS SeriesInstanceUID, 
                CONCAT(CONFIGURATION.`Value`,'\\', DICOM_FILES.rel_path) AS file_path
FROM          DICOM_FILES CROSS JOIN
                       CONFIGURATION
WHERE     ( CONFIGURATION.`Key`  = 'root_dir');

CREATE VIEW   CMOVE_IMAGE_VIEW 
AS
SELECT DISTINCT 
                      patient_name AS PatientsName, patient_id AS PatientID, 
				study_id AS StudyID, study_date_time AS StudyDate,
				accession_number AS AccessionNumber,
				study_instance_uid AS StudyInstanceUID, modality AS Modality,
				series_number AS SeriesNumber,
				series_instance_uid AS SeriesInstanceUID, 
				instance_number AS InstanceNumber,
                sop_class_uid AS SOPClassUID, sop_instance_uid AS SOPInstanceUID, 
                CONCAT(CONFIGURATION.`Value`,'\\', DICOM_FILES.rel_path) AS file_path
FROM          DICOM_FILES CROSS JOIN
                       CONFIGURATION
WHERE     ( CONFIGURATION.`Key`  = 'root_dir');

CREATE VIEW CFIND_ScheduledProcedureStepSequence_VIEW
AS
SELECT        
	id, 
	modality, 
	scheduled_station_ae_title AS ScheduledStationAETitle, 
	DATE(start_date_time) AS ScheduledProcedureStepStartDate, 
    perf_phys_name AS ScheduledPerformingPhysicianName, 
    req_proc_id AS ScheduledProcedureStepSequence, 
    TIME(start_date_time) AS ScheduledProcedureStepStartTime, 
    sched_proc_step_desc AS ScheduledProcedureStepDescription, 
    sched_proc_step_id AS ScheduledProcedureStepID, 
    NULL AS ScheduledPerformingPhysiciansName, 
    NULL AS ScheduledStationName, 
    NULL AS ScheduledProcedureStepLocation, 
    NULL AS PreMedication,
	start_date_time AS ScheduledProcedureStepStartDateTime, 
    DATE(end_date_time) AS ScheduledProcedureStepEndDate, 
    TIME(end_date_time) AS ScheduledProcedureStepEndTime, 
    end_date_time AS DateTime
FROM            
	SCHED_PROC_STEP;
     
CREATE VIEW STORAGE_COMMIT_FAILED_INSTANCES AS
    SELECT 
        STORAGE_COMMIT_REQUEST_INSTANCES.scm_id,
        STORAGE_COMMIT_REQUEST_INSTANCES.sop_class_uid,
        STORAGE_COMMIT_REQUEST_INSTANCES.sop_instance_uid
    FROM
        STORAGE_COMMIT_REQUEST_INSTANCES
            LEFT OUTER JOIN
        DICOM_FILES ON STORAGE_COMMIT_REQUEST_INSTANCES.sop_class_uid = DICOM_FILES.sop_class_uid
            AND STORAGE_COMMIT_REQUEST_INSTANCES.sop_instance_uid = DICOM_FILES.sop_instance_uid
    WHERE
        (DICOM_FILES.file_id IS NULL);

CREATE TABLE MPPS (
	id int NOT NULL AUTO_INCREMENT,
	sop_instance_uid nvarchar(64) NOT NULL,
	patient_name nvarchar(64) NULL,
	patient_id nvarchar(64) NULL,
	patient_sex nchar(1) NULL,
	patient_birth_date datetime NULL,
	pps_id nvarchar(16) NOT NULL,
	performed_station_ae_title nvarchar(16) NULL,
	performed_station_name nvarchar(16) NULL,
	performed_location nvarchar(16) NULL,
	pps_start_datetime datetime NULL,
	pps_status nvarchar(16) NOT NULL,
	pps_desc nvarchar(64) NULL,
	performed_procedure_type_desc nvarchar(64) NULL,
	pps_end_datetime datetime NULL,
	modality nvarchar(16) NULL,
	study_id nvarchar(16) NULL,
	rel_path nvarchar(400) NOT NULL,
    PRIMARY KEY (id  ASC));

CREATE TABLE MPPS_SPS(
	id int NOT NULL AUTO_INCREMENT,
	mpps_id int NOT NULL,
	study_instance_uid nvarchar(64) NOT NULL,
	accession_number nvarchar(16) NULL,
	requested_procedure_id nvarchar(50) NULL,
	requested_procedure_desc nvarchar(400) NULL,
	sps_id nvarchar(16) NULL,
	sps_desc nvarchar(400) NULL,
    PRIMARY KEY (id  ASC),
    FOREIGN KEY (mpps_id) REFERENCES MPPS(id) ON DELETE CASCADE);

CREATE VIEW AWAITING_REQUESTED_PROCEDURES_VIEW
AS
SELECT REQUESTED_PROCEDURE.id, REQUESTED_PROCEDURE.study_instance_uid, REQUESTED_PROCEDURE.req_proc_desc, REQUESTED_PROCEDURE.req_proc_id, REQUESTED_PROCEDURE.service_request_id, REQUESTED_PROCEDURE.patient_internal_id, 
         REQUESTED_PROCEDURE.RequestedProcedureCodeSequence
FROM  REQUESTED_PROCEDURE LEFT OUTER JOIN
         MPPS_SPS ON REQUESTED_PROCEDURE.study_instance_uid = MPPS_SPS.study_instance_uid
WHERE (MPPS_SPS.study_instance_uid IS NULL);


-- Change the MWL view to take only open procedures
Create VIEW CFIND_MWL_VIEW
AS
SELECT AWAITING_REQUESTED_PROCEDURES_VIEW.study_instance_uid AS StudyInstanceUID, AWAITING_REQUESTED_PROCEDURES_VIEW.req_proc_desc AS RequestedProcedureDescription, AWAITING_REQUESTED_PROCEDURES_VIEW.id AS ScheduledProcedureStepSequence, 
         AWAITING_REQUESTED_PROCEDURES_VIEW.req_proc_id AS RequestedProcedureID, SERVICE_REQUEST.accession_number AS AccessionNumber, SERVICE_REQUEST.referring_physicians_names AS ReferringPhysicianName, SERVICE_REQUEST.requesting_physician AS RequestingPhysician, N_PATIENT.patient_id AS PatientID, 
         N_PATIENT.patient_name AS PatientName, N_PATIENT.birth_date AS PatientBirthDate, N_PATIENT.sex AS PatientSex, NULL AS PatientWeight, NULL AS MedicalAlerts, NULL AS Allergies, NULL AS PregnancyStatus, NULL AS RequestedContrastAgent, NULL AS AdmissionID, NULL AS SpecialNeeds, NULL 
         AS CurrentPatientLocation, NULL AS PatientState, NULL AS RequestedProcedurePriority, NULL AS PatientTransportArrangements, NULL AS ConfidentialityConstraintOnPatientDataDescription, SERVICE_REQUEST.requesting_service
FROM  AWAITING_REQUESTED_PROCEDURES_VIEW INNER JOIN
         SERVICE_REQUEST ON AWAITING_REQUESTED_PROCEDURES_VIEW.service_request_id = SERVICE_REQUEST.id INNER JOIN
         N_PATIENT ON AWAITING_REQUESTED_PROCEDURES_VIEW.patient_internal_id = N_PATIENT.id;

CREATE VIEW MPPS_SPS_JOINED_VIEW
AS
SELECT MPPS.*, MPPS_SPS.study_instance_uid, MPPS_SPS.accession_number, MPPS_SPS.requested_procedure_id, MPPS_SPS.requested_procedure_desc, MPPS_SPS.sps_desc, 
                  MPPS_SPS.sps_id
FROM     MPPS INNER JOIN
                  MPPS_SPS ON MPPS.id = MPPS_SPS.mpps_id;

create view WORKLIST_VIEW
as
SELECT N_PATIENT.id AS patId, N_PATIENT.patient_id, N_PATIENT.patient_name, N_PATIENT.birth_date, N_PATIENT.sex, N_PATIENT.phone_number, N_PATIENT.address, 
                  REQUESTED_PROCEDURE.id AS reqProcId, REQUESTED_PROCEDURE.study_instance_uid, REQUESTED_PROCEDURE.req_proc_desc, REQUESTED_PROCEDURE.req_proc_id AS RPID, 
                  SERVICE_REQUEST.id AS servReqId, SERVICE_REQUEST.accession_number, SERVICE_REQUEST.referring_physicians_names, SERVICE_REQUEST.requesting_physician, 
                  SERVICE_REQUEST.requesting_service, SCHED_PROC_STEP.id AS shedStepId, SCHED_PROC_STEP.modality, SCHED_PROC_STEP.scheduled_station_ae_title, 
                  SCHED_PROC_STEP.start_date_time, SCHED_PROC_STEP.perf_phys_name, SCHED_PROC_STEP.sched_proc_step_desc, SCHED_PROC_STEP.sched_proc_step_id, 
                  MPPS_SPS_JOINED_VIEW.pps_start_datetime, MPPS_SPS_JOINED_VIEW.pps_end_datetime, MPPS_SPS_JOINED_VIEW.performed_station_ae_title, MPPS_SPS_JOINED_VIEW.pps_status, 
                  MPPS_SPS_JOINED_VIEW.pps_desc
FROM     REQUESTED_PROCEDURE INNER JOIN
                  N_PATIENT ON REQUESTED_PROCEDURE.patient_internal_id = N_PATIENT.id INNER JOIN
                  SERVICE_REQUEST ON REQUESTED_PROCEDURE.service_request_id = SERVICE_REQUEST.id INNER JOIN
                  SCHED_PROC_STEP ON REQUESTED_PROCEDURE.id = SCHED_PROC_STEP.req_proc_id LEFT OUTER JOIN
                  MPPS_SPS_JOINED_VIEW ON SCHED_PROC_STEP.sched_proc_step_id = MPPS_SPS_JOINED_VIEW.sps_id;            
            
CREATE VIEW   UNFINISHED_STORAGE_COMMIT_REQUESTS 
AS
SELECT      STORAGE_COMMIT_REQUEST.scm_id,  STORAGE_COMMIT_REQUEST.scm_source_ae,  DICOM_APPLICATIONS.host, 
                       DICOM_APPLICATIONS.port,  STORAGE_COMMIT_REQUEST.scm_transaction_uid
FROM          STORAGE_COMMIT_REQUEST INNER JOIN
                       DICOM_APPLICATIONS ON  STORAGE_COMMIT_REQUEST.scm_source_ae =  DICOM_APPLICATIONS.ae_title
WHERE      
((STORAGE_COMMIT_REQUEST.response_status IS NULL OR STORAGE_COMMIT_REQUEST.response_status <> 0) 
AND (STORAGE_COMMIT_REQUEST.response_retry_count < (SELECT  confvalue FROM CONFIGURATION_view 
	WHERE confkey = 'scm_max_retry'))
AND (STORAGE_COMMIT_REQUEST.update_time < (SELECT confvalue FROM CONFIGURATION_view
                              WHERE     (confkey = 'retry_seconds')) / 60 / 60 / 24 + NOW())
AND (STORAGE_COMMIT_REQUEST.scm_kind = 1))
OR ((STORAGE_COMMIT_REQUEST.response_status IS NULL OR STORAGE_COMMIT_REQUEST.response_status <> 0) 
AND (STORAGE_COMMIT_REQUEST.response_retry_count = 0) 
AND (STORAGE_COMMIT_REQUEST.scm_kind = 1));
                      
DELIMITER //
CREATE PROCEDURE SetLocalAE (IN ae_title nvarchar(16), IN portParam int)
BEGIN
	update CONFIGURATION set  `Value` = ae_title WHERE (`Key` = 'ae_title');
	update CONFIGURATION set  `Value` = portParam WHERE (`Key` = 'port');
END //
DELIMITER ;

DELIMITER //
CREATE PROCEDURE   SetVerbose (IN param1 nvarchar(50))
BEGIN
UPDATE CONFIGURATION SET `Value` = param1 WHERE (`Key` = 'Verbose');
END //
DELIMITER ;

DELIMITER //
CREATE PROCEDURE   SetDicomLog   (IN param1 nvarchar(50))
BEGIN
UPDATE CONFIGURATION SET `Value` = param1 WGetWebVirtualDirHERE (`Key`  = 'dicom_log');
END;

DELIMITER //
CREATE PROCEDURE   GetLocalAE(OUT ae_title nvarchar(16), OUT `port` int)
BEGIN
SELECT `Value` into ae_title  FROM CONFIGURATION WHERE ( `Key`  = 'ae_title');
SELECT `Value` into `port`  FROM CONFIGURATION WHERE ( `Key`  = 'port');
END //
DELIMITER ;

DELIMITER //
CREATE PROCEDURE GetFirstImage() 
BEGIN
	SELECT DICOM_FILES.rel_path 
	FROM  DICOM_FILES 
	GROUP BY 
		 DICOM_FILES.rel_path
    LIMIT 1;
END //
DELIMITER ;

DELIMITER //
CREATE PROCEDURE GetFirstImageInSeriesPath (IN SeriesUID nvarchar(64))
BEGIN
	SELECT DICOM_FILES.rel_path 
	FROM  DICOM_FILES 
	WHERE
		( DICOM_FILES.series_instance_uid  = SeriesUID)
	GROUP BY 
		 DICOM_FILES.rel_path
    LIMIT 1; 
END //
DELIMITER ;

DELIMITER //
CREATE PROCEDURE   GetFirstStudy() 
BEGIN
	SELECT 
		  DICOM_FILES.study_instance_uid 
	FROM  DICOM_FILES 
	GROUP BY 
		 DICOM_FILES.study_instance_uid 
         LIMIT 1;
END //
DELIMITER ;

DELIMITER //
CREATE PROCEDURE   GetImagePaths (IN SeriesUID nvarchar(64))
BEGIN
	SELECT 
		 DICOM_FILES.rel_path 
	FROM  DICOM_FILES 
	WHERE
		( DICOM_FILES.series_instance_uid  = SeriesUID)
	GROUP BY 
		 DICOM_FILES.rel_path ;
END //
DELIMITER ;

DELIMITER //
CREATE PROCEDURE   GetRootDir()
BEGIN
SELECT  `Value`  FROM CONFIGURATION WHERE ( `Key`  = 'root_dir');
END //
DELIMITER ;

DELIMITER //
CREATE PROCEDURE  GetSeries (IN StudyUID nvarchar(64))
BEGIN
	SELECT 
		 DICOM_FILES.series_instance_uid ,  DICOM_FILES.study_instance_uid 
	FROM  DICOM_FILES 
	WHERE
		( DICOM_FILES.study_instance_uid  = StudyUID)
	GROUP BY 
		 DICOM_FILES.series_instance_uid ,  DICOM_FILES.study_instance_uid; 
END //
DELIMITER ;

DELIMITER //
CREATE PROCEDURE GetStudies(IN PatientID nvarchar(64), IN PatientName nvarchar(64))
BEGIN
	SELECT DICOM_FILES.study_instance_uid, Count(*) AS NumberOfSeries 
	FROM  DICOM_FILES 
	WHERE
		( DICOM_FILES.patient_id  like PatientID AND  DICOM_FILES.patient_name  like PatientName)
	GROUP BY 
		 DICOM_FILES.study_instance_uid; 
END //
DELIMITER ;

DELIMITER //
CREATE PROCEDURE   GetWebVirtualDir() 
BEGIN
SELECT  `Value`  FROM CONFIGURATION WHERE ( `Key`  = 'web_virtual_dir');
END //
DELIMITER ;

DELIMITER //
CREATE PROCEDURE GetWebDir()
BEGIN
SELECT  `Value`  FROM CONFIGURATION WHERE ( `Key`  = 'web_dir');
END //
DELIMITER ;

DELIMITER //
CREATE PROCEDURE   CleanOldSCMs() 
BEGIN
	declare cleantime date;
	select cleantime = (NOW() - (SELECT `Value`
                            FROM           CONFIGURATION
                            WHERE      ( `Key`  = 'cleanup_seconds')) / 60 / 60 / 24);
	delete from storage_commit_request
	where (creation_time < @cleantime AND update_time < cleantime) OR
		  (creation_time < @cleantime AND (update_time IS NULL));
END //
DELIMITER ;

INSERT INTO   REQUESTED_PROCEDURE_CODES  ( RequestedProcedureDescription ) VALUES ('Physical examination');
INSERT INTO   REQUESTED_PROCEDURE_CODES  ( RequestedProcedureDescription ) VALUES ('Tests');
INSERT INTO   REQUESTED_PROCEDURE_CODES  ( RequestedProcedureDescription ) VALUES ('X-rays');
INSERT INTO   REQUESTED_PROCEDURE_CODES  ( RequestedProcedureDescription ) VALUES ('Scans');
INSERT INTO   REQUESTED_PROCEDURE_CODES  ( RequestedProcedureDescription ) VALUES ('Therapy');
INSERT INTO   REQUESTED_PROCEDURE_CODES  ( RequestedProcedureDescription ) VALUES ('Rehabilitation procedure');
INSERT INTO   REQUESTED_PROCEDURE_CODES  ( RequestedProcedureDescription ) VALUES ('Cosmetic procedure');


INSERT INTO  CONFIGURATION  ( `Key` , `Value` , Description ) VALUES ('ae_title', 'DSRSVC', 'Local ae title');
INSERT INTO  CONFIGURATION  ( `Key` , `Value` , Description ) VALUES ('port', '104', 'Listener port');
INSERT INTO  CONFIGURATION  ( `Key` , `Value` , Description ) VALUES ('retry_seconds', '180', 'The number of seconds to wait before retrying a failed task');
INSERT INTO  CONFIGURATION  ( `Key` , `Value` , Description ) VALUES ('root_dir', '\DSRSVC', 'Working file system directory');
INSERT INTO  CONFIGURATION  ( `Key` , `Value` , Description ) VALUES ('scm_max_retry', '3', 'The maximum number of retries for sending back commit responses');
INSERT INTO  CONFIGURATION  ( `Key` , `Value` , Description ) VALUES ('dicom_log', '0', 'When this value is 1, detailed logging is on');
INSERT INTO  CONFIGURATION  ( `Key` , `Value` , Description ) VALUES ('scm_on_same_assoc', '1', 'When this value is 1, storage_commit is performed on a new association');


