CREATE TABLE N_PATIENT (
	id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
	patient_id text(64) NULL,
	patient_message_id integer NULL,
	patient_name text(400) NULL,
	phone_number text(400) NULL,
	birth_date text(8) NULL,
	sex text(1) NULL,
	address text(400) NULL,
	ssn text(18) NULL
	);

CREATE TABLE REQUESTED_PROCEDURE (
	id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
	study_instance_uid text(64) NULL,
	req_proc_desc text(64) NULL,
	req_proc_id text(50) NULL,
	service_request_id integer NULL,
	patient_internal_id integer NULL,
	RequestedProcedureCodeSequence integer NULL
	);

CREATE TABLE SERVICE_REQUEST (
	id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
	accession_number text(50) NULL,
	referring_physicians_names text(400) NULL,
	requesting_physician text(400) NULL,
	requesting_service text(100) NULL);

CREATE TABLE REQUESTED_PROCEDURE_CODES (
    ReqProcCode INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
    RequestedProcedureDescription NVARCHAR(64) NOT NULL);

CREATE TABLE SCHED_PROC_STEP(
	id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
	modality text(16) NULL,
	scheduled_station_ae_title text(16) NULL,
	start_date_time text(23) NULL,
	end_date_time text(23) NULL,
	perf_phys_name text(250) NULL,
	sched_proc_step_desc text(64) NULL,
	sched_proc_step_id text(16) NULL,
	req_proc_id integer NOT NULL);

CREATE TABLE CONFIGURATION(
	Key text(50) NOT NULL,
	Value text(50) NOT NULL,
	Description text(400) NULL);

CREATE TABLE DICOM_FILES(
	file_id integer primary key asc NOT NULL,
	creation_time text(8) NOT NULL default (date('now')),
	update_time text(8) NULL,
	patient_name text(64) NOT NULL,
	patient_id text(64) NOT NULL,
	patient_sex text(1) NULL,
	patient_birth_date text(8) NULL,
	study_instance_uid text(64) NOT NULL,
	study_id text(16) NULL,
	study_date_time text(23) NULL,
	accession_number text(16) NULL,
	modality text(16) NULL,
	series_number text(16) NULL,
	series_instance_uid text(64) NOT NULL,
	instance_number text(16) NULL,
	sop_class_uid text(64) NOT NULL,
	sop_instance_uid text(64) NOT NULL,
	source_ae_title text(64) NULL,
	rel_path text(400) NOT NULL,
	task_id integer NULL);

CREATE TABLE MPPS (
	id integer primary key AUTOINCREMENT NOT NULL,
	sop_instance_uid text(64) NOT NULL,
	patient_name text(64) NULL,
	patient_id text(64) NULL,
	patient_sex text(1) NULL,
	patient_birth_date text(23) NULL,
	pps_id text(16) NOT NULL,
	performed_station_ae_title text(16) NULL,
	performed_station_name text(16) NULL,
	performed_location text(16) NULL,
	pps_start_datetime text(23) NULL,
	pps_status text(16) NOT NULL,
	pps_desc text(64) NULL,
	performed_procedure_type_desc text(64) NULL,
	pps_end_datetime text(23) NULL,
	modality text(16) NULL,
	study_id text(16) NULL,
	rel_path text(400) NOT NULL);

CREATE TABLE MPPS_SPS(
	id integer primary key AUTOINCREMENT NOT NULL,
	mpps_id integer NOT NULL,
	study_instance_uid text(64) NOT NULL,
	accession_number text(16) NULL,
	requested_procedure_id text(50) NULL,
	requested_procedure_desc text(400) NULL,
	sps_id text(16) NULL,
	sps_desc text(400) NULL,
    FOREIGN KEY(mpps_id) REFERENCES MPPS(id) on delete cascade);

CREATE TABLE   DICOM_APPLICATIONS (
	 id   integer  primary key AUTOINCREMENT NOT NULL,
	 alias   text (400) NULL,
	 ae_title   text (64) NOT NULL unique,
	 host   text (64) NULL,
	 port   integer  NULL); 

CREATE TABLE   STORAGE_COMMIT_REQUEST (
scm_id   integer  primary key AUTOINCREMENT NOT NULL,
creation_time text(8) NOT NULL default (date('now')),
update_time text(8) NULL,
scm_source_ae   text (64) NOT NULL,
scm_transaction_uid   text (64) NOT NULL unique,
tasks_queued   integer  NULL,
response_retry_count   integer  NOT NULL DEFAULT 0,
response_status   integer  NULL,
scm_kind   integer  NOT NULL DEFAULT 0,
task_id   integer  NULL);

CREATE TABLE STORAGE_COMMIT_REQUEST_INSTANCES (
scm_id   integer  NOT NULL,
sop_class_uid   text (64) NOT NULL unique,
sop_instance_uid   text (64) NOT NULL unique,
status   integer  NOT NULL DEFAULT 2,
failure_reason   integer  NOT NULL DEFAULT 0,
PRIMARY KEY (scm_id, sop_class_uid, sop_instance_uid),
FOREIGN KEY(scm_id) REFERENCES STORAGE_COMMIT_REQUEST (scm_id) ON UPDATE CASCADE ON DELETE CASCADE);

CREATE VIEW   STORAGE_COMMIT_SUCCEEDED_INSTANCES 
AS
SELECT      STORAGE_COMMIT_REQUEST_INSTANCES.scm_id,  STORAGE_COMMIT_REQUEST_INSTANCES.sop_class_uid, 
                       STORAGE_COMMIT_REQUEST_INSTANCES.sop_instance_uid
FROM          STORAGE_COMMIT_REQUEST_INSTANCES INNER JOIN
                       DICOM_FILES ON  STORAGE_COMMIT_REQUEST_INSTANCES.sop_class_uid =  DICOM_FILES.sop_class_uid AND 
                       STORAGE_COMMIT_REQUEST_INSTANCES.sop_instance_uid =  DICOM_FILES.sop_instance_uid;

CREATE VIEW STORAGE_COMMIT_FAILED_INSTANCES AS
    SELECT 
        STORAGE_COMMIT_REQUEST_INSTANCES.scm_id,
        STORAGE_COMMIT_REQUEST_INSTANCES.sop_class_uid,
        STORAGE_COMMIT_REQUEST_INSTANCES.sop_instance_uid
    FROM
        STORAGE_COMMIT_REQUEST_INSTANCES
            LEFT OUTER JOIN
        DICOM_FILES ON STORAGE_COMMIT_REQUEST_INSTANCES.sop_class_uid = DICOM_FILES.sop_class_uid
            AND STORAGE_COMMIT_REQUEST_INSTANCES.sop_instance_uid = DICOM_FILES.sop_instance_uid
    WHERE
        (DICOM_FILES.file_id IS NULL);

CREATE VIEW   UNFINISHED_STORAGE_COMMIT_REQUESTS 
AS
SELECT      STORAGE_COMMIT_REQUEST.scm_id,  STORAGE_COMMIT_REQUEST.scm_source_ae,  DICOM_APPLICATIONS.host, 
                       DICOM_APPLICATIONS.port,  STORAGE_COMMIT_REQUEST.scm_transaction_uid
FROM          STORAGE_COMMIT_REQUEST INNER JOIN
                       DICOM_APPLICATIONS ON  STORAGE_COMMIT_REQUEST.scm_source_ae =  DICOM_APPLICATIONS.ae_title
WHERE      
((STORAGE_COMMIT_REQUEST.response_status IS NULL OR STORAGE_COMMIT_REQUEST.response_status <> 0) 
AND (STORAGE_COMMIT_REQUEST.response_retry_count < (SELECT  value FROM CONFIGURATION 
	WHERE key = 'scm_max_retry'))
AND (STORAGE_COMMIT_REQUEST.update_time < (SELECT value FROM CONFIGURATION
                              WHERE     (key = 'retry_seconds')) / 60 / 60 / 24 + date('now'))
AND (STORAGE_COMMIT_REQUEST.scm_kind = 1))
OR ((STORAGE_COMMIT_REQUEST.response_status IS NULL OR STORAGE_COMMIT_REQUEST.response_status <> 0) 
AND (STORAGE_COMMIT_REQUEST.response_retry_count = 0) 
AND (STORAGE_COMMIT_REQUEST.scm_kind = 1));



CREATE VIEW AWAITING_REQUESTED_PROCEDURES_VIEW
AS
SELECT REQUESTED_PROCEDURE.id, REQUESTED_PROCEDURE.study_instance_uid, REQUESTED_PROCEDURE.req_proc_desc, REQUESTED_PROCEDURE.req_proc_id, REQUESTED_PROCEDURE.service_request_id, REQUESTED_PROCEDURE.patient_internal_id, 
         REQUESTED_PROCEDURE.RequestedProcedureCodeSequence
FROM  REQUESTED_PROCEDURE LEFT OUTER JOIN
         MPPS_SPS ON REQUESTED_PROCEDURE.study_instance_uid = MPPS_SPS.study_instance_uid
WHERE (MPPS_SPS.study_instance_uid IS NULL);

-- Change the MWL view to take only open procedures
Create VIEW CFIND_MWL_VIEW
AS
SELECT AWAITING_REQUESTED_PROCEDURES_VIEW.study_instance_uid AS StudyInstanceUID, AWAITING_REQUESTED_PROCEDURES_VIEW.req_proc_desc AS RequestedProcedureDescription, AWAITING_REQUESTED_PROCEDURES_VIEW.id AS ScheduledProcedureStepSequence, 
         AWAITING_REQUESTED_PROCEDURES_VIEW.req_proc_id AS RequestedProcedureID, SERVICE_REQUEST.accession_number AS AccessionNumber, SERVICE_REQUEST.referring_physicians_names AS ReferringPhysicianName, SERVICE_REQUEST.requesting_physician AS RequestingPhysician, N_PATIENT.patient_id AS PatientID, 
         N_PATIENT.patient_name AS PatientName, N_PATIENT.birth_date AS PatientBirthDate, N_PATIENT.sex AS PatientSex, NULL AS PatientWeight, NULL AS MedicalAlerts, NULL AS Allergies, NULL AS PregnancyStatus, NULL AS RequestedContrastAgent, NULL AS AdmissionID, NULL AS SpecialNeeds, NULL 
         AS CurrentPatientLocation, NULL AS PatientState, NULL AS RequestedProcedurePriority, NULL AS PatientTransportArrangements, NULL AS ConfidentialityConstraintOnPatientDataDescription, SERVICE_REQUEST.requesting_service
FROM  AWAITING_REQUESTED_PROCEDURES_VIEW INNER JOIN
         SERVICE_REQUEST ON AWAITING_REQUESTED_PROCEDURES_VIEW.service_request_id = SERVICE_REQUEST.id INNER JOIN
         N_PATIENT ON AWAITING_REQUESTED_PROCEDURES_VIEW.patient_internal_id = N_PATIENT.id;

CREATE VIEW MPPS_SPS_JOINED_VIEW
AS
SELECT MPPS.*, MPPS_SPS.study_instance_uid, MPPS_SPS.accession_number, MPPS_SPS.requested_procedure_id, MPPS_SPS.requested_procedure_desc, MPPS_SPS.sps_desc, 
                  MPPS_SPS.sps_id
FROM     MPPS INNER JOIN
                  MPPS_SPS ON MPPS.id = MPPS_SPS.mpps_id;

create view WORKLIST_VIEW
as
SELECT N_PATIENT.id AS patId, N_PATIENT.patient_id, N_PATIENT.patient_name, N_PATIENT.birth_date, N_PATIENT.sex, N_PATIENT.phone_number, N_PATIENT.address, 
                  REQUESTED_PROCEDURE.id AS reqProcId, REQUESTED_PROCEDURE.study_instance_uid, REQUESTED_PROCEDURE.req_proc_desc, REQUESTED_PROCEDURE.req_proc_id AS RPID, 
                  SERVICE_REQUEST.id AS servReqId, SERVICE_REQUEST.accession_number, SERVICE_REQUEST.referring_physicians_names, SERVICE_REQUEST.requesting_physician, 
                  SERVICE_REQUEST.requesting_service, SCHED_PROC_STEP.id AS shedStepId, SCHED_PROC_STEP.modality, SCHED_PROC_STEP.scheduled_station_ae_title, 
                  SCHED_PROC_STEP.start_date_time, SCHED_PROC_STEP.perf_phys_name, SCHED_PROC_STEP.sched_proc_step_desc, SCHED_PROC_STEP.sched_proc_step_id, 
                  MPPS_SPS_JOINED_VIEW.pps_start_datetime, MPPS_SPS_JOINED_VIEW.pps_end_datetime, MPPS_SPS_JOINED_VIEW.performed_station_ae_title, MPPS_SPS_JOINED_VIEW.pps_status, 
                  MPPS_SPS_JOINED_VIEW.pps_desc
FROM     REQUESTED_PROCEDURE INNER JOIN
                  N_PATIENT ON REQUESTED_PROCEDURE.patient_internal_id = N_PATIENT.id INNER JOIN
                  SERVICE_REQUEST ON REQUESTED_PROCEDURE.service_request_id = SERVICE_REQUEST.id INNER JOIN
                  SCHED_PROC_STEP ON REQUESTED_PROCEDURE.id = SCHED_PROC_STEP.req_proc_id LEFT OUTER JOIN
                  MPPS_SPS_JOINED_VIEW ON SCHED_PROC_STEP.sched_proc_step_id = MPPS_SPS_JOINED_VIEW.sps_id;

CREATE VIEW CFIND_ScheduledProcedureStepSequence_VIEW
AS
SELECT        
	id, 
	modality, 
	scheduled_station_ae_title AS ScheduledStationAETitle, 
	DATE(start_date_time) AS ScheduledProcedureStepStartDate, 
    perf_phys_name AS ScheduledPerformingPhysicianName, 
    req_proc_id AS ScheduledProcedureStepSequence, 
    TIME(start_date_time) AS ScheduledProcedureStepStartTime, 
    sched_proc_step_desc AS ScheduledProcedureStepDescription, 
    sched_proc_step_id AS ScheduledProcedureStepID, 
    NULL AS ScheduledPerformingPhysiciansName, 
    NULL AS ScheduledStationName, 
    NULL AS ScheduledProcedureStepLocation, 
    NULL AS PreMedication,
	start_date_time AS ScheduledProcedureStepStartDateTime, 
    DATE(end_date_time) AS ScheduledProcedureStepEndDate, 
    TIME(end_date_time) AS ScheduledProcedureStepEndTime, 
    end_date_time AS DateTime
FROM            
	SCHED_PROC_STEP;

CREATE VIEW CFIND_SERIES_VIEW
AS
SELECT DISTINCT 
                      patient_name AS PatientsName, patient_id AS PatientID, 
				study_id AS StudyID, study_date_time AS StudyDate,
				accession_number AS AccessionNumber,
				study_instance_uid AS StudyInstanceUID, modality AS Modality,
				series_number AS SeriesNumber,
				series_instance_uid AS SeriesInstanceUID
FROM         DICOM_FILES;

CREATE VIEW CFIND_IMAGE_VIEW
AS
SELECT DISTINCT 
                patient_name AS PatientsName, patient_id AS PatientID, 
				study_id AS StudyID, study_date_time AS StudyDate,
				accession_number AS AccessionNumber,
				study_instance_uid AS StudyInstanceUID, modality AS Modality,
				series_number AS SeriesNumber,
				series_instance_uid AS SeriesInstanceUID, 
				instance_number AS InstanceNumber,
                sop_class_uid AS SOPClassUID, sop_instance_uid AS SOPInstanceUID
FROM         DICOM_FILES;

CREATE VIEW CFIND_STUDY_VIEW
AS
SELECT DISTINCT patient_name AS PatientName, patient_id AS PatientID, study_id AS StudyID,
                DICOM_FILES.study_date_time AS StudyDate,
                DICOM_FILES.study_date_time AS StudyTime, accession_number AS AccessionNumber, study_instance_uid AS StudyInstanceUID, patient_sex AS PatientSex, patient_birth_date AS PatientBirthDate,
                (SELECT COUNT(SeriesInstanceUID) FROM CFIND_SERIES_VIEW WHERE (StudyInstanceUID = DICOM_FILES.study_instance_uid)) AS NumberOfStudyRelatedSeries,
                (SELECT COUNT(SOPInstanceUID) FROM    CFIND_IMAGE_VIEW WHERE (StudyInstanceUID = DICOM_FILES.study_instance_uid)) AS NumberOfStudyRelatedInstances,
                (SELECT DISTINCT Modality + '\' AS 'data()' FROM    CFIND_IMAGE_VIEW AS CFIND_IMAGE_VIEW_1 WHERE  (StudyInstanceUID = DICOM_FILES.study_instance_uid) ) AS ModalitiesInStudy
FROM    DICOM_FILES;

CREATE VIEW CFIND_PATIENT_VIEW
AS
SELECT DISTINCT 
    patient_name AS PatientName, 
	patient_id AS PatientID, 
	patient_sex AS PatientSex, 
	patient_birth_date AS PatientBirthDate
FROM         DICOM_FILES;

CREATE VIEW CMOVE_PATIENT_VIEW
AS
SELECT DISTINCT 
                      DICOM_FILES.patient_name AS PatientsName, DICOM_FILES.patient_id AS PatientID, 
                      CONFIGURATION.Value + '\' + DICOM_FILES.rel_path AS file_path
FROM         DICOM_FILES CROSS JOIN
                      CONFIGURATION
WHERE     (CONFIGURATION.Key = 'root_dir');

CREATE VIEW CMOVE_STUDY_VIEW
AS
SELECT DISTINCT 
                patient_name AS PatientsName, patient_id AS PatientID, 
				study_id AS StudyID, study_date_time AS StudyDate,
				accession_number AS AccessionNumber,
				study_instance_uid AS StudyInstanceUID, CONFIGURATION.Value + '\' + DICOM_FILES.rel_path AS file_path
FROM         DICOM_FILES CROSS JOIN
                      CONFIGURATION
WHERE     (CONFIGURATION.Key = 'root_dir');

CREATE VIEW CMOVE_SERIES_VIEW
AS
SELECT DISTINCT 
                      patient_name AS PatientsName, patient_id AS PatientID, 
				study_id AS StudyID, study_date_time AS StudyDate,
				accession_number AS AccessionNumber,
				study_instance_uid AS StudyInstanceUID, modality AS Modality,
				series_number AS SeriesNumber,
				series_instance_uid AS SeriesInstanceUID, 
                      CONFIGURATION.Value + '\' + DICOM_FILES.rel_path AS file_path
FROM         DICOM_FILES CROSS JOIN
                      CONFIGURATION
WHERE     (CONFIGURATION.Key = 'root_dir');

CREATE VIEW CMOVE_IMAGE_VIEW
AS
SELECT DISTINCT 
                      patient_name AS PatientsName, patient_id AS PatientID, 
				study_id AS StudyID, study_date_time AS StudyDate,
				accession_number AS AccessionNumber,
				study_instance_uid AS StudyInstanceUID, modality AS Modality,
				series_number AS SeriesNumber,
				series_instance_uid AS SeriesInstanceUID, 
				instance_number AS InstanceNumber,
                sop_class_uid AS SOPClassUID, sop_instance_uid AS SOPInstanceUID, 
                      CONFIGURATION.Value + '\' + DICOM_FILES.rel_path AS file_path
FROM         DICOM_FILES CROSS JOIN
                      CONFIGURATION
WHERE     (CONFIGURATION.Key = 'root_dir');

CREATE VIEW CONFIGURATION_VIEW
AS
SELECT      
	Key AS confkey,
	Value AS confvalue,
	Description AS confdescription
FROM            
	CONFIGURATION;

INSERT INTO N_PATIENT
           (id 
           ,patient_id
           ,patient_name
           ,birth_date
           ,sex)
     VALUES
           (123456789
           ,'123456789'
           ,'AB^CD'
           ,'1/1/1956'
           ,'M');

INSERT INTO SERVICE_REQUEST
           (id
           ,accession_number
           ,referring_physicians_names
           ,requesting_physician)
     VALUES
           (2121
           ,'RRZ000000000001'
           ,'Physician^Referring'
           ,'Physician^Requesting');




INSERT INTO REQUESTED_PROCEDURE
           (id
           ,study_instance_uid
           ,req_proc_desc
           ,service_request_id
           ,patient_internal_id)
     VALUES
           (1212
           ,'1.2.3.4.5.6.7.8'
           ,'Requested procedure description'
           ,2121
           ,123456789);

INSERT INTO SCHED_PROC_STEP
           (id
           ,modality
           ,scheduled_station_ae_title
           ,start_date_time
           ,end_date_time
           ,perf_phys_name
           ,sched_proc_step_desc
           ,sched_proc_step_id
           ,req_proc_id)
     VALUES
           (1234
           ,'CR'
           ,'TESTCR'
           ,'2016-02-01 16:30:00.000'
           ,'2016-02-01 17:00:00.000'
           ,'Performing Physician Name'
           ,'Secheduled Procedure Step Description'
           ,'SPSID1'
           ,1212);

/****** Insert data into the CONFIGURATION table ******/
INSERT INTO CONFIGURATION (Key,Value,Description) VALUES ('ae_title', 'DSRSVC', 'Local ae title');
INSERT INTO CONFIGURATION (Key,Value,Description) VALUES ('port', '104', 'Listener port');
INSERT INTO CONFIGURATION (Key,Value,Description) VALUES ('retry_seconds', '180', 'The number of seconds to wait before retrying a failed task');
INSERT INTO CONFIGURATION (Key,Value,Description) VALUES ('root_dir', '\DSRSVC', 'Working file system directory');
INSERT INTO CONFIGURATION (Key,Value,Description) VALUES ('scm_max_retry', '3', 'The maximum number of retries for sending back commit responses');
INSERT INTO CONFIGURATION (Key,Value,Description) VALUES ('dicom_log', '0', 'When this value is 1, detailed logging is on');
INSERT INTO  CONFIGURATION  ( Key , Value , Description ) VALUES ('scm_on_same_assoc', '1', 'When this value is 1, storage_commit is performed on a new association');




